/**
 * Copyright (c) 2005-2012 Fossil E-Commerce, 2005-2012 EBM WebSourcing, 2012-2015 Linagora
 * 
 * This program/library is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License as published by
 * the Free Software Foundation, either version 2.1 of the License, or (at your
 * option) any later version.
 * 
 * This program/library is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE. See the GNU Lesser General Public License
 * for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public License
 * along with this program/library; If not, see <http://www.gnu.org/licenses/>
 * for the GNU Lesser General Public License version 2.1.
 */
package org.ow2.petals.microkernel.jbi.component.context;

import static org.ow2.petals.microkernel.api.Constants.UNRESOLVED_INTERFACE;
import static org.ow2.petals.microkernel.api.container.ContainerService.COMPONENT_LOGGER_PREFIX;

import java.net.URISyntaxException;
import java.util.ArrayList;
import java.util.Collections;
import java.util.HashSet;
import java.util.List;
import java.util.MissingResourceException;
import java.util.Set;
import java.util.logging.Logger;

import javax.jbi.JBIException;
import javax.jbi.component.Component;
import javax.jbi.management.MBeanNames;
import javax.jbi.messaging.MessagingException;
import javax.jbi.servicedesc.ServiceEndpoint;
import javax.management.MBeanServer;
import javax.naming.InitialContext;
import javax.xml.namespace.QName;

import org.objectweb.fractal.api.NoSuchInterfaceException;
import org.objectweb.fractal.api.control.ContentController;
import org.objectweb.fractal.api.control.LifeCycleController;
import org.objectweb.fractal.util.Fractal;
import org.ow2.easywsdl.extensions.wsdl4complexwsdl.WSDL4ComplexWsdlReaders;
import org.ow2.easywsdl.extensions.wsdl4complexwsdl.api.WSDL4ComplexWsdlException;
import org.ow2.easywsdl.extensions.wsdl4complexwsdl.api.WSDL4ComplexWsdlReader;
import org.ow2.easywsdl.wsdl.api.Description;
import org.ow2.easywsdl.wsdl.api.Service;
import org.ow2.easywsdl.wsdl.api.WSDLException;
import org.ow2.petals.clientserverapi.jbi.messaging.registry.exception.RegistryException;
import org.ow2.petals.jbi.descriptor.original.generated.Jbi;
import org.ow2.petals.jbi.descriptor.original.generated.LinkType;
import org.ow2.petals.microkernel.api.container.ComponentContextCommunication;
import org.ow2.petals.microkernel.api.container.Installer;
import org.ow2.petals.microkernel.api.jbi.component.PetalsComponentContext;
import org.ow2.petals.microkernel.api.jbi.messaging.PetalsDeliveryChannel;
import org.ow2.petals.microkernel.api.jbi.messaging.RouterService;
import org.ow2.petals.microkernel.api.jbi.security.SecurityContext;
import org.ow2.petals.microkernel.api.jbi.servicedesc.PetalsServiceEndpoint;
import org.ow2.petals.microkernel.api.server.FractalHelper;
import org.ow2.petals.microkernel.jbi.messaging.exchange.DeliveryChannelImpl;
import org.ow2.petals.microkernel.server.exception.InvalidCompBasedArchiException;
import org.ow2.petals.microkernel.util.ParameterCheckHelper;
import org.w3c.dom.Document;
import org.w3c.dom.DocumentFragment;

import com.ebmwebsourcing.easycommons.log.LoggingUtil;

/**
 * @author Rafael Marins - Fossil E-Commerce
 * @author wjoseph - EBM WebSourcing
 */
public class ComponentContextImpl implements PetalsComponentContext {

    protected PetalsDeliveryChannel deliveryChannel;

    /**
     * references of registered external serviceEndpoints
     */
    protected Set<ServiceEndpoint> externalEndpoints;

    /**
     * references of registered internal serviceEndpoints
     */
    protected Set<ServiceEndpoint> internalEndpoints;

    protected Component jbiComponent;

    /**
     * The component context communication (can be either an installer or a
     * component life cycle)
     */
    protected final ComponentContextCommunication componentContextCommunication;

    private final LoggingUtil logger = new LoggingUtil(
            java.util.logging.Logger
                    .getLogger(org.ow2.petals.microkernel.jbi.component.context.Constants.FRACTAL_COMPONENT_LOGGER_NAME));

    /**
     * The life-cycle controller of the fractal component 'SystemRecovery'
     */
    private final LifeCycleController systemRecoveryComponentLC;

    private SecurityContext securityContext;

    /**
     * Constructor of the ComponentContextImpl class
     * 
     * @throws InvalidCompBasedArchiException
     * 
     */
    public ComponentContextImpl(final ComponentContextCommunication componentContextCommunication,
            final org.objectweb.fractal.api.Component jbiMngtComposite) throws InvalidCompBasedArchiException {
        super();
        this.componentContextCommunication = componentContextCommunication;
        this.internalEndpoints = new HashSet<ServiceEndpoint>();
        this.externalEndpoints = new HashSet<ServiceEndpoint>();
        try {
            final ContentController jbiMngtCompositeCC = Fractal.getContentController(jbiMngtComposite);
            final org.objectweb.fractal.api.Component systemRecoveyComponent = FractalHelper
                    .getRecursiveComponentByName(jbiMngtCompositeCC, FractalHelper.SYSTEMRECOVERY_COMPONENT);
            this.systemRecoveryComponentLC = Fractal.getLifeCycleController(systemRecoveyComponent);
        } catch (final NoSuchInterfaceException e) {
            throw new InvalidCompBasedArchiException(e);
        }
    }

    @Override
    public ServiceEndpoint activateEndpoint(QName serviceName, String endpointName)
            throws JBIException {
        this.logger.start();

        ParameterCheckHelper.isNullParameter(serviceName, "serviceName");
        ParameterCheckHelper.isNullParameter(serviceName, "endpointName");

        final QName service = serviceName;
        final String endpoint = endpointName;

        Document doc = this.getComponent().getServiceDescription(new ServiceEndpoint() {
            public DocumentFragment getAsReference(QName operationName) {
                return null;
            }

            public String getEndpointName() {
                return endpoint;
            }

            public QName[] getInterfaces() {
                return new QName[] {};
            }

            public QName getServiceName() {
                return service;
            }

        });

        // switch the classloader to the container one
        // CAUTION : this must be done AFTER getServiceDescription is called
        ClassLoader componentClassLoader = Thread.currentThread().getContextClassLoader();
        Thread.currentThread().setContextClassLoader(this.getClass().getClassLoader());


        final List<QName> interfaces;
        if (doc != null) {
            final List<QName> extractedInterfaces = this.getInterfacesFromComponentDescription(doc,
                    serviceName, endpointName);
            if (!extractedInterfaces.isEmpty()) {
                interfaces = Collections.unmodifiableList(extractedInterfaces);
            } else {
                final QName defaultInterface = getDefaultInterface(serviceName);
                this.logger
                        .warning("No interface has been found in the component service description, set default to "
                                + defaultInterface);
                interfaces = Collections.singletonList(defaultInterface);
            }
        } else {
            final QName defaultInterface = getDefaultInterface(serviceName);
            this.logger.warning("No description has been found for service '" + service.toString()
                    + "', endpoint name '" + endpointName + "'. Interface set default to "
                    + defaultInterface);
            interfaces = Collections.singletonList(defaultInterface);
        }

        final boolean isRecoveringInProgress = LifeCycleController.STARTED.equals(this.systemRecoveryComponentLC
                .getFcState());

        final ServiceEndpoint se;
        try {
            this.logger.debug("Activating the new created endpoint in the registry "
                    + (isRecoveringInProgress ? "with" : "without") + " overwriting.");
            se = this.componentContextCommunication.getEndpointRegistry().activateEndpoint(serviceName, endpointName,
                    interfaces, doc, this.componentContextCommunication.getAddress(), isRecoveringInProgress);
        } catch (final RegistryException e) {
            throw new JBIException(e);
        } finally {
            Thread.currentThread().setContextClassLoader(componentClassLoader);
        }
        this.internalEndpoints.add(se);
        this.logger.end();
        return se;
    }

    @Override
    public void clear() {
        this.deliveryChannel = null;
        this.jbiComponent = null;
    }

    private final static QName getDefaultInterface(final QName serviceName) {
        return new QName(serviceName.getNamespaceURI(), UNRESOLVED_INTERFACE
                + serviceName.getLocalPart());
    }

    @Override
    public void deactivateEndpoint(javax.jbi.servicedesc.ServiceEndpoint endpoint)
            throws JBIException {
        this.logger.start();

        ParameterCheckHelper.isNullParameter(endpoint, "endpoint");

        // switch the classloader to the container one
        ClassLoader componentClassLoader = Thread.currentThread().getContextClassLoader();
        Thread.currentThread().setContextClassLoader(this.getClass().getClassLoader());
        try {
            try {
                this.componentContextCommunication.getEndpointRegistry().deactivateEndpoint(endpoint.getEndpointName(),
                        endpoint.getServiceName());
            } catch (final RegistryException e) {
                throw new JBIException(e);
            }
        } finally {
            Thread.currentThread().setContextClassLoader(componentClassLoader);
        }
        this.internalEndpoints.remove(endpoint);
        this.logger.end();
    }

    @Override
    public void deregisterAllEndpoints() throws JBIException {
        this.logger.start();

        // switch the classloader to the container one
        ClassLoader componentClassLoader = Thread.currentThread().getContextClassLoader();
        Thread.currentThread().setContextClassLoader(this.getClass().getClassLoader());
        try {
            for (ServiceEndpoint se : this.internalEndpoints) {
                this.componentContextCommunication.getEndpointRegistry().deactivateEndpoint(
                        se.getEndpointName(), se.getServiceName());
            }

            for (ServiceEndpoint se : this.externalEndpoints) {
                this.componentContextCommunication.getEndpointRegistry()
                        .deregisterExternalEndpoint(se.getEndpointName(), se.getServiceName());
            }

        } catch (final RegistryException e) {
            throw new JBIException(e);
        } finally {
            this.internalEndpoints.clear();
            this.externalEndpoints.clear();
            Thread.currentThread().setContextClassLoader(componentClassLoader);

            this.logger.end();
        }
    }

    @Override
    public void deregisterExternalEndpoint(ServiceEndpoint externalEndpoint) throws JBIException {
        this.logger.start();

        ParameterCheckHelper.isNullParameter(externalEndpoint, "externalEndpoint");

        try {
            this.componentContextCommunication.getEndpointRegistry().deregisterExternalEndpoint(
                    externalEndpoint.getEndpointName(), externalEndpoint.getServiceName());
            this.externalEndpoints.remove(externalEndpoint);
        } catch (final RegistryException e) {
            throw new JBIException(e);
        } finally {
            this.logger.end();
        }
    }

    @Override
    public PetalsServiceEndpoint getAddress() {
        return this.componentContextCommunication.getAddress();
    }

    @Override
    public Component getComponent() {
        return this.jbiComponent;
    }

    @Override
    public String getComponentName() {
        return this.componentContextCommunication.getJBIDescriptor().getComponent()
                .getIdentification().getName();
    }

    @Override
    public PetalsDeliveryChannel getDeliveryChannel()
            throws MessagingException {
        this.logger.start();

        if (this.deliveryChannel == null) {
            this.deliveryChannel = new DeliveryChannelImpl(this);
        } else if (!this.deliveryChannel.isOpened()) {
            this.deliveryChannel = new DeliveryChannelImpl(this);
        } else {
            // Do not throw an exception to be TCK and other containers
            // compliant
            // throw new MessagingException("Delivery Channel already open");
        }

        this.logger.end();
        return this.deliveryChannel;
    }
    
    @Override
    public PetalsDeliveryChannel getDeliveryChannelInstance() {
        return this.deliveryChannel;
    }

    @Override
    public ServiceEndpoint getEndpoint(QName service, String name) {
        this.logger.start();

        ParameterCheckHelper.isNullParameter(service, "service");
        ParameterCheckHelper.isNullParameter(name, "name");

        // switch the classloader to the container one
        ClassLoader componentClassLoader = Thread.currentThread().getContextClassLoader();
        Thread.currentThread().setContextClassLoader(this.getClass().getClassLoader());
        ServiceEndpoint result = null;
        try {
            result = this.componentContextCommunication.getEndpointRegistry().getEndpoint(service,
                    name);
        } catch (Exception e) {
            this.logger.warning(e.getMessage());
        } finally {
            Thread.currentThread().setContextClassLoader(componentClassLoader);
        }
        this.logger.end();
        return result;
    }

    @Override
    public Document getEndpointDescriptor(final ServiceEndpoint endpoint) throws JBIException {
        this.logger.start();
        Document result = null;
        ParameterCheckHelper.isNullParameter(endpoint, "endpoint");

        if (endpoint instanceof PetalsServiceEndpoint) {
            this.logger.debug("Just get the description from the service endpoint");

            // FIXME = Do not get it but just check if it exists...
            if (this.getEndpoint(endpoint.getServiceName(), endpoint.getEndpointName()) != null) {

                // switch the classloader to the container one
                ClassLoader componentClassLoader = Thread.currentThread().getContextClassLoader();
                Thread.currentThread().setContextClassLoader(this.getClass().getClassLoader());
                try {
                    result = ((PetalsServiceEndpoint) endpoint).getDescription();
                } finally {
                    Thread.currentThread().setContextClassLoader(componentClassLoader);
                }

                if (result == null) {
                    this.logger.debug("Can not find the service description in the service endpoint");
                }
            } else {
                throw new JBIException("Invalid endpoint reference: it does not exists");
            }
        } else {
            throw new JBIException(
                    "Invalid endpoint reference: must be of type " + PetalsServiceEndpoint.class.getName());
        }
        this.logger.end();
        return result;
    }

    @Override
    public ServiceEndpoint[] getEndpoints(QName interfaceName) {
        this.logger.start();

        // switch the classloader to the container one
        ClassLoader componentClassLoader = Thread.currentThread().getContextClassLoader();
        Thread.currentThread().setContextClassLoader(this.getClass().getClassLoader());
        ServiceEndpoint[] result = null;
        try {
            result = this.componentContextCommunication.getEndpointRegistry()
                    .getInternalEndpointsForInterface(interfaceName, LinkType.STANDARD);
        } catch (Exception e) {
            this.logger.warning(e.getMessage());
        } finally {
            Thread.currentThread().setContextClassLoader(componentClassLoader);
        }

        this.logger.end();
        return result;
    }

    @Override
    public ServiceEndpoint[] getEndpointsForService(QName serviceName) {
        this.logger.start();

        ParameterCheckHelper.isNullParameter(serviceName, "serviceName");

        // switch the classloader to the container one
        ClassLoader componentClassLoader = Thread.currentThread().getContextClassLoader();
        Thread.currentThread().setContextClassLoader(this.getClass().getClassLoader());
        ServiceEndpoint[] result = null;
        try {
            result = this.componentContextCommunication.getEndpointRegistry()
                    .getInternalEndpointsForService(serviceName, LinkType.STANDARD);
        } catch (RegistryException e) {
            this.logger.warning(e.getMessage());
        } finally {
            Thread.currentThread().setContextClassLoader(componentClassLoader);
        }

        this.logger.end();
        return result;
    }

    @Override
    public ServiceEndpoint[] getExternalEndpoints(QName interfaceName) {
        this.logger.start();

        ParameterCheckHelper.isNullParameter(interfaceName, "interfaceName");

        // switch the classloader to the container one
        ClassLoader componentClassLoader = Thread.currentThread().getContextClassLoader();
        Thread.currentThread().setContextClassLoader(this.getClass().getClassLoader());
        ServiceEndpoint[] result = null;
        try {
            result = this.componentContextCommunication.getEndpointRegistry()
                    .getExternalEndpointsForInterface(interfaceName);
        } catch (RegistryException e) {
            this.logger.warning(e.getMessage());
        } finally {
            Thread.currentThread().setContextClassLoader(componentClassLoader);
        }

        this.logger.end();
        return result;
    }

    @Override
    public ServiceEndpoint[] getExternalEndpointsForService(QName serviceName) {
        this.logger.start();

        ParameterCheckHelper.isNullParameter(serviceName, "serviceName");

        // switch the classloader to the container one
        ClassLoader componentClassLoader = Thread.currentThread().getContextClassLoader();
        Thread.currentThread().setContextClassLoader(this.getClass().getClassLoader());
        ServiceEndpoint[] result = null;
        try {
            result = this.componentContextCommunication.getEndpointRegistry()
                    .getExternalEndpointsForService(serviceName);
        } catch (RegistryException e) {
            this.logger.warning(e.getMessage());
        } finally {
            Thread.currentThread().setContextClassLoader(componentClassLoader);
        }

        this.logger.end();
        return result;
    }

    @Override
    public String getInstallRoot() {
        this.logger.call();

        return this.componentContextCommunication.getInstallationRoot();
    }

    public Jbi getJbi() {
        return this.componentContextCommunication.getJBIDescriptor();
    }

    @Override
    public Logger getLogger() {
        return this.componentContextCommunication.getLogger();
    }

    @Override
    public Logger getLogger(String suffix, String resourceBundleName)
            throws MissingResourceException, JBIException {
        this.logger.start();

        String loggerName = this.getComponentName();

        // append the suffix to the logger Name
        if (suffix == null) {
            throw new IllegalArgumentException("suffix cannot be null");
        } else {
            if (suffix.length() > 0) {
                loggerName += "." + suffix;
            }
        }

        // get a new logger or retrieve it
        final Logger newLogger;
        try {
            newLogger = Logger.getLogger(COMPONENT_LOGGER_PREFIX + "." + loggerName, resourceBundleName);
        } catch (final IllegalArgumentException e) {
            // note: the message of e contains the names of conflicting resources bundles names
            throw new JBIException("Logger already exists for suffix " + loggerName, e);
        }

        this.logger.end();
        return newLogger;
    }

    @Override
    public MBeanNames getMBeanNames() {
        this.logger.call();

        return this.componentContextCommunication.getAdminService().getMBeanNames();
    }

    @Override
    public MBeanServer getMBeanServer() {
        this.logger.call();

        return this.componentContextCommunication.getAdminService().getMBeanServer();
    }

    @Override
    public InitialContext getNamingContext() {
        this.logger.call();

        return this.componentContextCommunication.getInitialContext();
    }

    @Override
    public RouterService getRouter() {
        return this.componentContextCommunication.getRouterService();
    }

    @Override
    public SecurityContext getSecurityContext() {
        return this.securityContext;
    }

    @Override
    public Object getTransactionManager() {
        this.logger.call();

        // this is not implemented
        return null;
    }

    @Override
    public String getWorkspaceRoot() {
        this.logger.call();

        return this.componentContextCommunication.getWorkspaceRoot();
    }

    @Override
    public void registerExternalEndpoint(ServiceEndpoint externalEndpoint) throws JBIException {
        this.logger.start();

        ParameterCheckHelper.isNullParameter(externalEndpoint, "externalEndpoint");

        // switch the classloader to the container one
        ClassLoader componentClassLoader = Thread.currentThread().getContextClassLoader();
        Thread.currentThread().setContextClassLoader(this.getClass().getClassLoader());
        try {
            this.componentContextCommunication.getEndpointRegistry().registerExternalEndpoint(
                    externalEndpoint);
        } catch (final RegistryException e) {
            throw new JBIException(e);
        } finally {
            Thread.currentThread().setContextClassLoader(componentClassLoader);
        }
        this.externalEndpoints.add(externalEndpoint);

        this.logger.end();
    }

    @Override
    public ServiceEndpoint resolveEndpointReference(DocumentFragment epr) {
        this.logger.start();

        ParameterCheckHelper.isNullParameter(epr, "epr");

        javax.jbi.servicedesc.ServiceEndpoint result = null;

        // switch the classloader to the container one
        ClassLoader componentClassLoader = Thread.currentThread().getContextClassLoader();
        Thread.currentThread().setContextClassLoader(this.getClass().getClassLoader());
        try {

            for (Installer installer : this.componentContextCommunication.getContainerService().getInstallers()
                    .values()) {
                result = installer.getComponent().resolveEndpointReference(epr);
                if (result != null) {
                    break;
                }
            }
        } finally {
            Thread.currentThread().setContextClassLoader(componentClassLoader);
        }

        this.logger.end();
        return result;

    }

    @Override
    public void setComponent(Component component) {
        this.jbiComponent = component;
    }

    public void setDeliveryChannel(PetalsDeliveryChannel deliveryChannel) {
        this.deliveryChannel = deliveryChannel;
    }

    @Override
    public void setSecurityContext(SecurityContext securityContext) {
        this.securityContext = securityContext;
    }

    private List<QName> getInterfacesFromComponentDescription(Document doc, QName serviceName,
            String endpointName) {
        List<QName> result = new ArrayList<QName>();
        try {
            WSDL4ComplexWsdlReader wsdl4ComplexWsdlReader = null;
            try {
                wsdl4ComplexWsdlReader = WSDL4ComplexWsdlReaders.takeWSDL4ComplexWsdlReader();
                Description wsdlDescription = wsdl4ComplexWsdlReader.read(doc);

                if ((wsdlDescription != null) && (wsdlDescription.getServices() != null)) {
                    Service ss = wsdlDescription.getService(serviceName);
                    if (ss != null) {
                        if (ss.getEndpoint(endpointName) != null) {
                            if ((ss.getInterface() != null)
                                    && (ss.getInterface().getQName() != null)) {
                                result.add(ss.getInterface().getQName());
                            } else {
                                this.logger
                                        .warning("No interface for endpoint in component description for service '"
                                                + ss.getQName() + "'");
                            }
                        } else {
                            this.logger
                                    .warning("No endpoint in component description for service '"
                                            + serviceName + "'");
                        }
                    } else {
                        this.logger.warning("No such service in component description '"
                                + serviceName + "'");
                    }
                }
            } finally {
                if (wsdl4ComplexWsdlReader != null) {
                    WSDL4ComplexWsdlReaders.releaseWSDL4ComplexWsdlReader(wsdl4ComplexWsdlReader);
                }
            }
        } catch (WSDL4ComplexWsdlException e) {
            this.logger.warning(e);
        } catch (WSDLException e) {
            this.logger.warning(e);
        } catch (URISyntaxException e) {
            this.logger.warning(e);
        }
        return result;
    }

}
