/**
 * Copyright (c) 2005-2012 EBM WebSourcing, 2012-2015 Linagora
 * 
 * This program/library is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License as published by
 * the Free Software Foundation, either version 2.1 of the License, or (at your
 * option) any later version.
 * 
 * This program/library is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE. See the GNU Lesser General Public License
 * for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public License
 * along with this program/library; If not, see <http://www.gnu.org/licenses/>
 * for the GNU Lesser General Public License version 2.1.
 */
package org.ow2.petals.microkernel.jbi.messaging.routing.module;

import java.io.IOException;
import java.lang.management.ManagementFactory;
import java.lang.management.MemoryMXBean;
import java.lang.management.MemoryNotificationInfo;
import java.lang.management.MemoryPoolMXBean;
import java.lang.management.MemoryType;
import java.sql.SQLException;
import java.util.ArrayDeque;
import java.util.Deque;
import java.util.Iterator;
import java.util.Map;
import java.util.Timer;
import java.util.TimerTask;
import java.util.concurrent.atomic.AtomicLong;
import java.util.concurrent.locks.LockSupport;
import java.util.logging.Logger;

import javax.management.Notification;
import javax.management.NotificationEmitter;
import javax.management.NotificationListener;

import org.objectweb.fractal.fraclet.annotations.Component;
import org.objectweb.fractal.fraclet.annotations.Interface;
import org.objectweb.fractal.fraclet.annotations.Lifecycle;
import org.objectweb.fractal.fraclet.annotations.Requires;
import org.objectweb.fractal.fraclet.types.Step;
import org.ow2.petals.jbi.messaging.exchange.PetalsMessageExchange;
import org.ow2.petals.microkernel.api.jbi.component.PetalsComponentContext;
import org.ow2.petals.microkernel.api.jbi.messaging.RouterService;
import org.ow2.petals.microkernel.api.jbi.messaging.RoutingException;
import org.ow2.petals.microkernel.api.jbi.messaging.exchange.PersistedPetalsMessageExchange;
import org.ow2.petals.microkernel.api.jbi.servicedesc.PetalsServiceEndpoint;
import org.ow2.petals.microkernel.api.system.persistence.PersistenceService;
import org.ow2.petals.microkernel.jbi.messaging.routing.module.flow.PriorityModifier;
import org.ow2.petals.microkernel.transport.util.TransportSendContext;

import com.ebmwebsourcing.easycommons.log.LoggingUtil;

/**
 * The module orderer the exchange according to their priority, proposing a Priority Queue mechanism. 
 * <p>
 * It is possible to reassign a priority by using the
 * {@link PriorityModifier} Interface.
 * </p>
 * 
 * @author Roland Naudin - EBM WebSourcing
 */
@Component(provides = { @Interface(name = "service", signature = SenderModule.class),
        @Interface(name = "prioritymodifier", signature = PriorityModifier.class) })
public class PriorityOrdererModule implements SenderModule, NotificationListener, PriorityModifier {

    /**
     * Inner class to process priorities.
     * 
     * @author rnaudin
     * 
     */
    private class PriorityProcessor extends Thread {
        
        private final static String THREAD_NAME = "Router - Priority orderer module - Priority Processor";

        private boolean isRunning = true;
        
        public PriorityProcessor() {
            super(THREAD_NAME);
        }

        private void releaseExchanges(final Deque<PetalsMessageExchange> exchangePriority, final int count) {
            int counter = count;
            while (counter > 0) {
                final PetalsMessageExchange exchange = exchangePriority.pollLast();
                if (exchange != null) {
                    final Object o = exchange.getProperty(WAIT_NOTIFY_PROPERTY);
                    synchronized (o) {
                        // unset the property to denote that the processing is finished
                        exchange.setProperty(WAIT_NOTIFY_PROPERTY, null);
                        // and notify to release the wait
                        o.notifyAll();
                    }
                    PriorityOrdererModule.this.pendingExchanges.decrementAndGet();
                    counter--;
                } else {
                    counter = 0;
                }
            }
        }

        @Override
        public void run() {

            while (this.isRunning) {

                // wait for incoming messages
                if (PriorityOrdererModule.this.pendingExchanges.get() == 0) {
                    if (PriorityOrdererModule.this.timeoutManagerTimer != null) {
                        PriorityOrdererModule.this.timeoutManagerTimer.cancel();
                        PriorityOrdererModule.this.timeoutManagerTimer = null;
                    }
                    LockSupport.park(this);
                }

                // priority 3
                synchronized (PriorityOrdererModule.this.exchangePriority3) {
                    releaseExchanges(PriorityOrdererModule.this.exchangePriority3, 3);
                }

                // priority 2
                synchronized (PriorityOrdererModule.this.exchangePriority2) {
                    releaseExchanges(PriorityOrdererModule.this.exchangePriority2, 2);
                }

                // priority 1
                synchronized (PriorityOrdererModule.this.exchangePriority1) {
                    releaseExchanges(PriorityOrdererModule.this.exchangePriority1, 1);
                }
            }
        }

    }

    /**
     * Inner class to handle the timeout for exchanges pending in the 0 priority
     * queue.
     * 
     * @author rnaudin
     * 
     */
    private class TimeoutManager extends TimerTask {

        @Override
        public void run() {

            final long currentTime = System.currentTimeMillis();

            for (final PetalsMessageExchange exchange : PriorityOrdererModule.this.exchangePriority0) {
                final Long deadline = (Long) exchange.getProperty(RouterService.PROPERTY_ROUTER_TIMEOUT_DEADLINE);
                if (deadline != null && currentTime > deadline) {
                    PriorityOrdererModule.this.log.debug("Exchange with Id '" + exchange.getExchangeId()
                            + "' is in send timeout, let's move it to priority 3");
                    try {
                        PriorityOrdererModule.this.modifyPriority(exchange, (short) 3);
                    } catch (RoutingException e) {
                        // the exchange has been already moved from 0 priority
                        // queue, do nothing
                    }
                }
            }
        }

    }

    /**
     * Logger wrapper.
     */
    private final LoggingUtil log = new LoggingUtil(
            Logger.getLogger(Constants.FRACTAL_COMPONENT_LOGGER_NAME_PRIORITY_ORDER));

    /**
     * The Persistence Fractal Service
     */
    @Requires(name = "persistence")
    private PersistenceService persistenceService;

    /**
     * The priority property to set in message exchange.
     */
    public static final String PROPERTY_ROUTER_PRIORITY = "org.petals.ow2.router.priority";

    /**
     * The period to run the Timeout Manager task
     */
    public static final long TIMEOUT_MANAGER_PERIOD = 1000;

    /**
     * The property to store the object used for wait/notify synchronisation.
     * 
     * We don't use the object itself as it is already used in NioTransporterImpl and LocalTransporterImpl
     */
    private static final String WAIT_NOTIFY_PROPERTY = "org.ow2.petals.microkernel.jbi.messaging.routing.module.PriorityOrdererModule.WaitNotifyObject";

    /**
     * Waiting queue
     */
    private Deque<PetalsMessageExchange> exchangePriority0;

    /**
     * Low priority queue
     */
    private Deque<PetalsMessageExchange> exchangePriority1;

    /**
     * Medium priority queue
     */
    private Deque<PetalsMessageExchange> exchangePriority2;

    /**
     * High priority queue
     */
    private Deque<PetalsMessageExchange> exchangePriority3;

    /**
     * Pending exchanges in the priority queues
     */
    private AtomicLong pendingExchanges;

    /**
     * The priority processor
     */
    private PriorityProcessor priorityProcessor;

    /**
     * The timeout manager timer
     */
    private Timer timeoutManagerTimer;

    @Override
    public void electEndpoints(final Map<PetalsServiceEndpoint, TransportSendContext> electedEndpoints,
            final PetalsComponentContext sourceComponentContext, final PetalsMessageExchange exchange)
                    throws RoutingException {

        String priority = (String) exchange.getProperty(PROPERTY_ROUTER_PRIORITY);
        if (priority != null) {
            final Object o = new Object();
            assert exchange.getProperty(WAIT_NOTIFY_PROPERTY) == null;
            exchange.setProperty(WAIT_NOTIFY_PROPERTY, o);
            synchronized (o) {
                if ("3".equals(priority)) {
                    synchronized (this.exchangePriority3) {
                        this.log.debug("Add Exchange with Id '" + exchange.getExchangeId() + "' to priority 3");
                        this.exchangePriority3.add(exchange);
                    }
                } else if ("2".equals(priority)) {
                    synchronized (this.exchangePriority2) {
                        this.log.debug("Add Exchange with Id '" + exchange.getExchangeId() + "' to priority 2");
                        this.exchangePriority2.add(exchange);
                    }
                } else if ("1".equals(priority)) {
                    synchronized (this.exchangePriority1) {
                        this.log.debug("Add Exchange with Id '" + exchange.getExchangeId() + "' to priority 1");
                        this.exchangePriority1.add(exchange);
                    }
                } else if ("0".equals(priority)) {
                    synchronized (this.exchangePriority0) {
                        this.log.debug("Add Exchange with Id '" + exchange.getExchangeId() + "' to priority 0");
                        this.exchangePriority0.add(exchange);
                    }
                }

                if (this.pendingExchanges.getAndIncrement() == 0) {
                    this.timeoutManagerTimer = new Timer("Router - Priority orderer module - Timeout manager");
                    this.timeoutManagerTimer.schedule(new TimeoutManager(), TIMEOUT_MANAGER_PERIOD,
                            TIMEOUT_MANAGER_PERIOD);
                    LockSupport.unpark(this.priorityProcessor);
                }

                try {
                    // add the while in case there is a spurious wake-up (see documentation of wait)
                    // it should have been set to null in releaseExchanges, which is the condition
                    // we are waiting for!
                    while (exchange.getProperty(WAIT_NOTIFY_PROPERTY) != null) {
                        o.wait();
                    }
                } catch (InterruptedException e) {
                    boolean decrement = false;
                    synchronized (exchangePriority3) {
                        decrement = this.exchangePriority3.remove(exchange);
                    }
                    if (!decrement) {
                        synchronized (exchangePriority2) {
                            decrement = this.exchangePriority2.remove(exchange);
                        }
                    }
                    if (!decrement) {
                        synchronized (exchangePriority1) {
                            decrement = this.exchangePriority1.remove(exchange);
                        }
                    }
                    if (!decrement) {
                        synchronized (exchangePriority0) {
                            decrement = this.exchangePriority0.remove(exchange);
                        }
                    }
                    if (decrement) {
                        this.pendingExchanges.decrementAndGet();
                    }
                    throw new RoutingException("Message Exchange with Id '" + exchange.getExchangeId()
                            + "' has been interrupted from priority ordering", e);
                } finally {
                    exchange.setProperty(PROPERTY_ROUTER_PRIORITY, null);
                    exchange.setProperty(WAIT_NOTIFY_PROPERTY, null);
                }
            }

        }
    }

    @Override
    public void handleNotification(Notification notification, Object handback) {
        if (MemoryNotificationInfo.MEMORY_THRESHOLD_EXCEEDED.equals(notification.getType())) {
            this.log.info("Memory threshold reached, persist some pending exchange if any");
            int pendingMessage = this.exchangePriority0.size() + this.exchangePriority1.size()
                    + this.exchangePriority2.size() + this.exchangePriority3.size();
            int counter = 0;

            // target 50% of pending message to persist

            // first all the exchange in waiting state
            synchronized (this.exchangePriority0) {
                Iterator<PetalsMessageExchange> iterator = this.exchangePriority0.iterator();
                while (iterator.hasNext()) {
                    final PetalsMessageExchange mex = iterator.next();

                    if (!(mex instanceof PersistedPetalsMessageExchange)) {
                        this.log.warning(
                                "The message is not a PersistedPetalsMessageExchange: " + mex.getClass().getName());
                        continue;
                    }

                    final PersistedPetalsMessageExchange exchange = (PersistedPetalsMessageExchange) mex;
                    try {
                        if (!exchange.isMessageExchangeStored()) {
                            exchange.persistExchange(this.persistenceService);
                            this.log.debug("Persist Message Exchange with Id '" + exchange.getExchangeId() + "'");
                            counter++;
                        }
                    } catch (SQLException e) {
                        this.log.warning(
                                "Failed to persist Message Exchange with Id '" + exchange.getExchangeId() + "'", e);
                    } catch (IOException e) {
                        this.log.warning(
                                "Failed to store a monitored exchange with Id '" + exchange.getExchangeId() + "'", e);
                    }
                }
            }

            if (counter < pendingMessage / 2) {
                // persist the half of low priority
                synchronized (this.exchangePriority1) {
                    final Iterator<PetalsMessageExchange> iterator = this.exchangePriority1.iterator();
                    int limit = this.exchangePriority1.size() / 2;
                    while (iterator.hasNext() && limit > 0) {

                        final PetalsMessageExchange mex = iterator.next();

                        if (!(mex instanceof PersistedPetalsMessageExchange)) {
                            this.log.warning(
                                    "The message is not a PersistedPetalsMessageExchange: " + mex.getClass().getName());
                            continue;
                        }

                        final PersistedPetalsMessageExchange exchange = (PersistedPetalsMessageExchange) mex;

                        try {
                            if (!exchange.isMessageExchangeStored()) {
                                exchange.persistExchange(this.persistenceService);
                                this.log.debug("Persist Message Exchange with Id '" + exchange.getExchangeId() + "'");
                                counter++;
                                limit++;
                            }
                        } catch (SQLException e) {
                            this.log.warning(
                                    "Failed to persist Message Exchange with Id '" + exchange.getExchangeId() + "'", e);
                        } catch (IOException e) {
                            this.log.warning(
                                    "Failed to persist Message Exchange with Id '" + exchange.getExchangeId() + "'", e);
                        }
                    }
                }
            }

            if (counter < pendingMessage / 2) {
                // persist the half of medium priority
                synchronized (this.exchangePriority2) {
                    Iterator<PetalsMessageExchange> iterator = this.exchangePriority2.iterator();
                    int limit = this.exchangePriority2.size() / 2;
                    while (iterator.hasNext() && limit > 0) {

                        final PetalsMessageExchange mex = iterator.next();

                        if (!(mex instanceof PersistedPetalsMessageExchange)) {
                            this.log.warning(
                                    "The message is not a PersistedPetalsMessageExchange: " + mex.getClass().getName());
                            continue;
                        }

                        final PersistedPetalsMessageExchange exchange = (PersistedPetalsMessageExchange) mex;
                        try {
                            if (!exchange.isMessageExchangeStored()) {
                                exchange.persistExchange(this.persistenceService);
                                this.log.debug("Persist Message Exchange with Id '" + exchange.getExchangeId() + "'");
                                counter++;
                                limit++;
                            }
                        } catch (SQLException e) {
                            this.log.warning(
                                    "Failed to persist Message Exchange with Id '" + exchange.getExchangeId() + "'", e);
                        } catch (IOException e) {
                            this.log.warning(
                                    "Failed to persist Message Exchange with Id '" + exchange.getExchangeId() + "'", e);
                        }
                    }
                }
            }

            if (counter < pendingMessage / 2) {
                // persist the half of high priority
                synchronized (this.exchangePriority3) {
                    Iterator<PetalsMessageExchange> iterator = this.exchangePriority3.iterator();
                    int limit = this.exchangePriority3.size() / 2;
                    while (iterator.hasNext() && limit > 0) {

                        final PetalsMessageExchange mex = iterator.next();

                        if (!(mex instanceof PersistedPetalsMessageExchange)) {
                            this.log.warning(
                                    "The message is not a PersistedPetalsMessageExchange: " + mex.getClass().getName());
                            continue;
                        }

                        final PersistedPetalsMessageExchange exchange = (PersistedPetalsMessageExchange) mex;

                        try {
                            if (!exchange.isMessageExchangeStored()) {
                                exchange.persistExchange(this.persistenceService);
                                this.log.debug("Persist Message Exchange with Id '" + exchange.getExchangeId() + "'");
                                counter++;
                                limit++;
                            }
                        } catch (SQLException e) {
                            this.log.warning(
                                    "Failed to persist Message Exchange with Id '" + exchange.getExchangeId() + "'", e);
                        } catch (IOException e) {
                            this.log.warning(
                                    "Failed to persist Message Exchange with Id '" + exchange.getExchangeId() + "'", e);
                        }
                    }
                }
            }
        }

    }

    @Override
    public void modifyPriority(final PetalsMessageExchange exchange, final short priority) throws RoutingException {
        this.log.call("Exchange Id '" + exchange.getExchangeId() + "' - Priority " + priority);

        boolean found = false;

        synchronized (this.exchangePriority3) {
            found = this.exchangePriority3.remove(exchange);
        }
        if (!found) {
            synchronized (this.exchangePriority2) {
                found = this.exchangePriority2.remove(exchange);
            }
        }
        if (!found) {
            synchronized (this.exchangePriority1) {
                found = this.exchangePriority1.remove(exchange);
            }
        }
        if (!found) {
            synchronized (this.exchangePriority0) {
                found = this.exchangePriority0.remove(exchange);
            }
        }
        if (!found) {
            throw new RoutingException(
                    "The Message exchange with Id '" + exchange.getExchangeId() + "' is no more in priority queues");
        } else {
            if (priority == 3) {
                synchronized (this.exchangePriority3) {
                    this.exchangePriority3.add(exchange);
                }
            } else if (priority == 2) {
                synchronized (this.exchangePriority2) {
                    this.exchangePriority2.add(exchange);
                }
            } else if (priority == 1) {
                synchronized (this.exchangePriority1) {
                    this.exchangePriority1.add(exchange);
                }
            } else if (priority == 0) {
                synchronized (this.exchangePriority0) {
                    this.exchangePriority0.add(exchange);
                }
            } else {
                throw new RoutingException("Priority " + priority + " is not supported");
            }
        }
    }

    /**
     * Start the Fractal component
     * 
     * @throws RoutingException
     */
    @Lifecycle(step = Step.START)
    public void start() {
        this.log.start();

        this.pendingExchanges = new AtomicLong();
        // TODO Uses ConcurrentLinkedDeque with Java 7 instead of locking everywhere
        // (note: we must verify this is really equivalent for traversal and size computing)
        this.exchangePriority0 = new ArrayDeque<PetalsMessageExchange>();
        this.exchangePriority1 = new ArrayDeque<PetalsMessageExchange>();
        this.exchangePriority2 = new ArrayDeque<PetalsMessageExchange>();
        this.exchangePriority3 = new ArrayDeque<PetalsMessageExchange>();

        for (MemoryPoolMXBean memoryPoolMXBean : ManagementFactory.getMemoryPoolMXBeans()) {
            if (MemoryType.HEAP.equals(memoryPoolMXBean.getType())
                    && memoryPoolMXBean.isUsageThresholdSupported()) {
                long maxUsage = memoryPoolMXBean.getUsage().getMax();
                final Double threshold = new Double(maxUsage * 0.8);
                memoryPoolMXBean.setUsageThreshold(threshold.longValue());
            }
        }
        final MemoryMXBean memoryMbean = ManagementFactory.getMemoryMXBean();
        ((NotificationEmitter) memoryMbean).addNotificationListener(this, null, null);

        this.priorityProcessor = new PriorityProcessor();
        this.priorityProcessor.start();

        this.log.end();
    }

    /**
     * Stop the Fractal component
     */
    @Lifecycle(step = Step.STOP)
    public void stop() {
        this.log.start();

        // the waiting threads should have been interrupted already by their
        // component owners, no need to treat these interruption here

        this.priorityProcessor.isRunning = false;
        LockSupport.unpark(this.priorityProcessor);
        if (this.timeoutManagerTimer != null) {
            this.timeoutManagerTimer.cancel();
        }

        this.log.end();
    }
}
