/**
 * Copyright (c) 2013-2015 Linagora
 * 
 * This program/library is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License as published by
 * the Free Software Foundation, either version 2.1 of the License, or (at your
 * option) any later version.
 * 
 * This program/library is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE. See the GNU Lesser General Public License
 * for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public License
 * along with this program/library; If not, see <http://www.gnu.org/licenses/>
 * for the GNU Lesser General Public License version 2.1.
 */

package org.ow2.petals.microkernel.server;

import java.util.Iterator;
import java.util.Properties;
import java.util.ServiceLoader;
import java.util.logging.Logger;

import org.objectweb.fractal.api.Component;
import org.objectweb.fractal.api.Interface;
import org.objectweb.fractal.api.NoSuchInterfaceException;
import org.objectweb.fractal.api.control.BindingController;
import org.objectweb.fractal.api.control.ContentController;
import org.objectweb.fractal.api.control.IllegalBindingException;
import org.objectweb.fractal.api.control.IllegalContentException;
import org.objectweb.fractal.api.control.IllegalLifeCycleException;
import org.objectweb.fractal.api.factory.InstantiationException;
import org.objectweb.fractal.fraclet.annotations.Lifecycle;
import org.objectweb.fractal.fraclet.annotations.Requires;
import org.objectweb.fractal.fraclet.extensions.Controller;
import org.objectweb.fractal.fraclet.types.Step;
import org.objectweb.fractal.util.Fractal;
import org.ow2.petals.basisapi.exception.PetalsException;
import org.ow2.petals.clientserverapi.server.exception.PetalsCompositeCtrlException;
import org.ow2.petals.microkernel.admin.PetalsAdminServiceImpl;
import org.ow2.petals.microkernel.api.admin.PetalsAdminInterface;
import org.ow2.petals.microkernel.api.communication.topology.TopologyService;
import org.ow2.petals.microkernel.api.communication.topology.exception.LockAlreadyAcquiredException;
import org.ow2.petals.microkernel.api.communication.topology.exception.TopologyException;
import org.ow2.petals.microkernel.api.configuration.ConfigurationService;
import org.ow2.petals.microkernel.api.container.ContainerService;
import org.ow2.petals.microkernel.api.container.SharedLibraryBean;
import org.ow2.petals.microkernel.api.container.exception.ComponentLifecycleNotFoundException;
import org.ow2.petals.microkernel.api.container.exception.ContainerServiceException;
import org.ow2.petals.microkernel.api.container.exception.InstallerNotFoundException;
import org.ow2.petals.microkernel.api.implementation.exception.MutableImplException;
import org.ow2.petals.microkernel.api.jbi.management.ManagementException;
import org.ow2.petals.microkernel.api.jbi.messaging.registry.EndpointRegistry;
import org.ow2.petals.microkernel.api.server.FractalHelper;
import org.ow2.petals.microkernel.container.ContainerController;
import org.ow2.petals.microkernel.container.exception.ContainerCtrlException;
import org.ow2.petals.microkernel.extension.PreExtensionsManager;
import org.ow2.petals.microkernel.jbi.management.recovery.SystemRecoveryService;
import org.ow2.petals.microkernel.server.exception.BindingInterfaceException;
import org.ow2.petals.microkernel.server.exception.CreateComponentException;
import org.ow2.petals.microkernel.server.exception.InvalidCompBasedArchiException;
import org.ow2.petals.microkernel.server.exception.NoRegistryImplException;
import org.ow2.petals.microkernel.server.exception.TooManyRegistryImplException;

import com.ebmwebsourcing.easycommons.log.LoggingUtil;

/**
 * The controller implementation of the Fractal composite 'Petals'
 * 
 * @author Christophe DENEUX - Linagora
 * 
 */
@org.objectweb.fractal.fraclet.annotations.Component(provides = @org.objectweb.fractal.fraclet.annotations.Interface(name = "service", signature = PetalsCompositeController.class))
public class PetalsCompositeControllerImpl implements PetalsCompositeController {

    /**
     * The configuration service
     */
    @Requires(name = "configuration")
    private ConfigurationService configurationService;

    /**
     * The pre-extension manager service
     */
    @Requires(name = "preExtensionsMgr")
    private PreExtensionsManager preExtensionsMgr;

    @Controller
    private Component comp;

    /**
     * The composite 'Petals'
     */
    private Component petalsComposite;

    /**
     * The content controller of the composite 'Petals'
     */
    private ContentController petalsCompositeCC;

    /**
     * The registry implementation
     */
    private Class<?> registryImplDef;

    private final LoggingUtil log = new LoggingUtil(Logger.getLogger(PetalsCompositeController.COMPONENT_LOGGER_NAME));

    /**
     * <p>
     * Starts the composite controller.
     * </p>
     * <p>
     * When starting, the composite controller will create the following Fractal
     * component, bind interfaces and start it:
     * <ol>
     * <li>the composite 'Communication',</li>
     * <li>the composite 'System',</li>
     * <li>the composite 'ExtensionManager',</li>
     * <li>the composite 'JBI-Management',</li>
     * <li>the component
     * 'org.ow2.petals.microkernel.admin.PetalsAdminServiceImpl',</li>
     * <li>the composite 'JBI-Messaging',</li>
     * <li>the composite 'Transporter',</li>
     * <li>the composite 'Container'.</li>
     * </ol>
     * </p>
     * 
     * @throws PetalsCompositeCtrlException
     *             An error occurs during the instantiation of a composite or a
     *             component.
     * @throws InvalidCompBasedArchiException
     *             The component based architecture is invalid.
     */
    @Lifecycle(step = Step.START)
    public void start() throws PetalsCompositeCtrlException, InvalidCompBasedArchiException {
        this.log.start();

        try {
            try {
                this.petalsComposite = FractalHelper.getParentComponent(this.comp);
                this.petalsCompositeCC = Fractal.getContentController(petalsComposite);

                // Creates composites
                this.log.debug("Creating composites");

                this.log.debug("Creates, binds and starts the composite " + FractalHelper.SYSTEM_COMPOSITE);
                final Component systemComposite = this.createAndAddCompositeSystem();
                final Component configuration = FractalHelper.getComponentByName(this.petalsCompositeCC,
                        FractalHelper.CONFIGURATION_COMPONENT);
                final Interface configurationServiceItf = (Interface) configuration
                        .getFcInterface(ConfigurationService.FRACTAL_SRV_ITF_NAME);
                this.bindCompositeSystemItf(systemComposite, configurationServiceItf);
                this.log.debug("Starts composite " + FractalHelper.SYSTEM_COMPOSITE);
                FractalHelper.startComponent(systemComposite);
                this.log.debug("Composite '" + FractalHelper.SYSTEM_COMPOSITE + "' created, bound and started.");

                this.log.debug("Creates, binds and starts the composite "
                        + FractalHelper.COMMUNICATION_COMPOSITE);
                final Component communicationComposite = this.createAndAddComponent(
                        FractalHelper.COMMUNICATION_COMPOSITE, FractalHelper.COMMUNICATION_COMPOSITE,
                        this.petalsComposite);
                final Interface registryServiceItf = (Interface) systemComposite
                        .getFcInterface(PetalsCompositeController.SYSTEM_REGISTRY_FRACTAL_SRV_ITF_NAME);
                final Component petalsCompositeController = FractalHelper.getComponentByName(this.petalsCompositeCC,
                        FractalHelper.PETALS_COMPOSITE_CTRL_COMPONENT);
                final Interface petalsCompositeCtrlItf = (Interface) petalsCompositeController
                        .getFcInterface(PetalsCompositeController.FRACTAL_SRV_ITF_NAME);
                this.bindCompositeCommunicationItf(communicationComposite, configurationServiceItf, registryServiceItf,
                        petalsCompositeCtrlItf);
                this.log.debug("Starts composite " + FractalHelper.COMMUNICATION_COMPOSITE);
                FractalHelper.startComponent(communicationComposite);
                this.log.debug("Composite '" + FractalHelper.COMMUNICATION_COMPOSITE
                        + "' created, bound and started.");

                // Before starting other components, we update the topology with the dynamic topology store in the
                // registry
                final Component topologyComponent = FractalHelper.getRecursiveComponentByName(this.petalsCompositeCC,
                        FractalHelper.TOPOLOGY_COMPONENT);
                if (topologyComponent == null) {
                    throw new InvalidCompBasedArchiException("Cannot retrieve the component '"
                            + FractalHelper.TOPOLOGY_COMPONENT + "'");
                } else {
                    final TopologyService topologyService = (TopologyService) topologyComponent
                            .getFcInterface(TopologyService.FRACTAL_SRV_ITF_NAME);
                    try {
                        topologyService.loadTopology();
                    } catch (final LockAlreadyAcquiredException e) {
                        throw new PetalsCompositeCtrlException(e);
                    } catch (TopologyException e) {
                        throw new PetalsCompositeCtrlException(e);
                    }
                }

                this.log.debug("Creates remaining composites/components");
                final Component extensionsManagerComposite = this.createAndAddComponent(
                        FractalHelper.POST_EXTENSIONSMANAGER_COMPOSITE, FractalHelper.POST_EXTENSIONSMANAGER_COMPOSITE,
                        this.petalsComposite);
                final Component jbiManagementComposite = this.createAndAddComponent(
                        FractalHelper.JBI_MANAGEMENT_COMPOSITE, FractalHelper.JBI_MANAGEMENT_COMPOSITE,
                        this.petalsComposite);
                final Component petalsAdminServiceComponent = this.createAndAddComponent(PetalsAdminServiceImpl.class,
                        FractalHelper.PETALSADMIN_COMPONENT,
                        this.petalsComposite);
                final Component jbiMessagingComposite = this.createAndAddComponent(
                        FractalHelper.JBI_MESSAGING_COMPOSITE, FractalHelper.JBI_MESSAGING_COMPOSITE,
                        this.petalsComposite);
                final Component transporterComposite = this.createAndAddComponent(
                        FractalHelper.TRANSPORTER_COMPOSITE, FractalHelper.TRANSPORTER_COMPOSITE, this.petalsComposite);
                this.log.debug("Composites created");

                // Bind interfaces
                this.log.debug("Binds interfaces of remaining composites/components");
                final Interface topologyServiceItf = (Interface) communicationComposite
                        .getFcInterface(PetalsCompositeController.COMMUNICATION_TOPOLOGY_FRACTAL_SRV_ITF_NAME);
                final Interface jmxServiceItf = (Interface) communicationComposite
                        .getFcInterface(PetalsCompositeController.COMMUNICATION_JMX_FRACTAL_SRV_ITF_NAME);
                final Interface jndiServiceItf = (Interface) communicationComposite
                        .getFcInterface(PetalsCompositeController.COMMUNICATION_JNDI_FRACTAL_SRV_ITF_NAME);
                final Interface systemStateServiceItf = (Interface) systemComposite
                        .getFcInterface(PetalsCompositeController.SYSTEM_SYSTEMSTATE_FRACTAL_SRV_ITF_NAME);
                final Interface classloaderServiceItf = (Interface) systemComposite
                        .getFcInterface(PetalsCompositeController.SYSTEM_CLASSLOADER_FRACTAL_SRV_ITF_NAME);
                final Interface repositoryServiceItf = (Interface) systemComposite
                        .getFcInterface(PetalsCompositeController.SYSTEM_REPOSITORY_FRACTAL_SRV_ITF_NAME);
                final Interface persistenceServiceItf = (Interface) systemComposite
                        .getFcInterface(PetalsCompositeController.SYSTEM_PERSISTENCE_FRACTAL_SRV_ITF_NAME);
                final Interface deploymentServiceItf = (Interface) jbiManagementComposite
                        .getFcInterface(PetalsCompositeController.JBIMANAGEMENT_DEPLOYMENT_FRACTAL_SRV_ITF_NAME);
                final Interface installationServiceItf = (Interface) jbiManagementComposite
                        .getFcInterface(PetalsCompositeController.JBIMANAGEMENT_INSTALLATION_FRACTAL_SRV_ITF_NAME);
                final Interface componentInstallationServiceItf = (Interface) jbiManagementComposite
                        .getFcInterface(PetalsCompositeController.JBIMANAGEMENT_COMPONENTINSTALLATION_FRACTAL_SRV_ITF_NAME);
                final Interface atomicDeploymentServiceItf = (Interface) jbiManagementComposite
                        .getFcInterface(PetalsCompositeController.JBIMANAGEMENT_ATOMICDEPLOYMENT_FRACTAL_SRV_ITF_NAME);
                final Interface adminServiceItf = (Interface) jbiManagementComposite
                        .getFcInterface(PetalsCompositeController.JBIMANAGEMENT_ADMIN_FRACTAL_SRV_ITF_NAME);
                final Interface containerServiceItf = (Interface) jbiManagementComposite
                        .getFcInterface(PetalsCompositeController.JBIMANAGEMENT_CONTAINER_FRACTAL_SRV_ITF_NAME);
                final Interface routerServiceItf = (Interface) jbiMessagingComposite
                        .getFcInterface(PetalsCompositeController.JBIMESSAGING_ROUTER_FRACTAL_SRV_ITF_NAME);
                final Interface transportListenerServiceItf = (Interface) jbiMessagingComposite
                        .getFcInterface(PetalsCompositeController.JBIMESSAGING_TRANSPORTLISTENER_FRACTAL_SRV_ITF_NAME);
                final Interface installationExtensionsManagerItf = (Interface) extensionsManagerComposite
                        .getFcInterface(PetalsCompositeController.POSTEXTENSIONSMANAGER_INSTALLATIONEXTENSIONSMANAGER_FRACTAL_SRV_ITF_NAME);
                final Interface petalsAdminServiceItf = (Interface) petalsAdminServiceComponent
                        .getFcInterface(PetalsAdminInterface.FRACTAL_SRV_ITF_NAME);
                final Interface transporterLocalServiceItf = (Interface) transporterComposite
                        .getFcInterface(PetalsCompositeController.TRANSPORTER_TRANSPORTERLOCAL_FRACTAL_SRV_ITF_NAME);
                final Interface transporterTcpServiceItf = (Interface) transporterComposite
                        .getFcInterface(PetalsCompositeController.TRANSPORTER_TRANSPORTERTCP_FRACTAL_SRV_ITF_NAME);

                this.bindCompositePostExtensionsManagerItf(extensionsManagerComposite,
                        configurationServiceItf, topologyServiceItf, jmxServiceItf,
                        systemStateServiceItf, registryServiceItf, deploymentServiceItf,
                        installationServiceItf, componentInstallationServiceItf,
                        atomicDeploymentServiceItf, adminServiceItf, containerServiceItf, petalsAdminServiceItf);
                this.bindCompositeJbiManagementItf(jbiManagementComposite, configurationServiceItf, jmxServiceItf,
                        systemStateServiceItf, classloaderServiceItf, repositoryServiceItf, registryServiceItf,
                        routerServiceItf, installationExtensionsManagerItf, jndiServiceItf);
                this.bindComponentPetalsAdminService(petalsAdminServiceComponent,
                        configurationServiceItf, topologyServiceItf, deploymentServiceItf,
                        installationServiceItf, registryServiceItf);
                this.bindCompositeJbiMessagingItf(jbiMessagingComposite, configurationServiceItf,
                        topologyServiceItf, jndiServiceItf, jmxServiceItf, persistenceServiceItf,
                        transporterLocalServiceItf, transporterTcpServiceItf, registryServiceItf);
                this.bindCompositeTransporterItf(transporterComposite, configurationServiceItf,
                        topologyServiceItf, transportListenerServiceItf);
                this.log.debug("Interfaces of composites bound");

                // Starts composites
                this.log.debug("Starts remaining composites/component");

                this.log.debug("Starts composite " + FractalHelper.POST_EXTENSIONSMANAGER_COMPOSITE);
                FractalHelper.startComponent(extensionsManagerComposite);

                this.log.debug("Starts composite " + FractalHelper.JBI_MANAGEMENT_COMPOSITE);
                FractalHelper.startComponent(jbiManagementComposite);

                this.log.debug("Starts composite " + FractalHelper.PETALSADMIN_COMPONENT);
                FractalHelper.startComponent(petalsAdminServiceComponent);

                this.startMessagingSubsystem();

                this.log.debug("Composites started");

            } catch (final NoSuchInterfaceException e) {
                throw new InvalidCompBasedArchiException(e);
            } catch (final IllegalLifeCycleException e) {
                throw new InvalidCompBasedArchiException(e);
            } finally {
                this.log.end();
            }
        } catch (final InvalidCompBasedArchiException e) {
            this.log.error(e.getMessage(), e);
            throw e;
        } catch (final PetalsCompositeCtrlException e) {
            this.log.error(e.getMessage(), e);
            throw e;
        } catch (final Exception e) {
            this.log.error("An unexpected error occurs.", e);
            throw new PetalsCompositeCtrlException(e);
        }
    }

    private final Component createAndAddComponent(final String componentDefinition, final String componentName,
            final Component parentComposite) throws CreateComponentException {
        // Create the Fractal composite from its name
        final Component composite;
        try {
            composite = FractalHelper.createCompositeComponent(componentDefinition);
            addComponentToPetals(composite, componentName, parentComposite);
        } catch (final InstantiationException e) {
            throw new CreateComponentException(componentName, e);
        }
        return composite;

    }

    private final Component createAndAddComponent(final Class<?> componentClass, final String componentName,
            final Component parentComposite) throws CreateComponentException {
        // Create the Fractal component from its class
        final Component component;
        try {
            component = FractalHelper.createPrimitiveComponent(componentClass);
            addComponentToPetals(component, componentName, parentComposite);
        } catch (final InstantiationException e) {
            throw new CreateComponentException(componentName, e);
        }
        return component;
    }

    /**
     * Create a composite or component into the composite {@value FractalHelper#PETALS_COMPOSITE}, from its name.
     * 
     * @param componentDefinition
     *            Definition of the composite or component to create
     * @param componentName
     *            Name of the composite or component in the Fractal environment.
     * @param parentComposite
     *            The composite in which the composite or component will be created.
     * @return The composite or component created
     * @throws CreateComponentException
     *             An error occurs during the instantiation of the composite
     */
    private final void addComponentToPetals(final Component component, final String componentName,
            final Component parentComposite) throws CreateComponentException {
        this.log.start(componentName);
        try {
            // Add the composite (or component) in the parent component
            FractalHelper.addComponent(component, parentComposite, null, componentName);
        } catch (final NoSuchInterfaceException e) {
            throw new CreateComponentException(componentName, e);
        } catch (final IllegalContentException e) {
            throw new CreateComponentException(componentName, e);
        } catch (final IllegalLifeCycleException e) {
            throw new CreateComponentException(componentName, e);
        } catch (final IllegalBindingException e) {
            throw new CreateComponentException(componentName, e);
        } finally {
            this.log.end(componentName);
        }
    }

    /**
     * <p>
     * Create the composite {@value FractalHelper#SYSTEM_COMPOSITE} into the
     * composite {@value FractalHelper#PETALS_COMPOSITE}.
     * </p>
     * <p>
     * The composite creation is done with following ordered steps:
     * <ol>
     * <li>load the core of the composite 'System' described by a Fractal ADL
     * file,</li>
     * <li>the dummy registry implementation is removed. This dummy registration
     * implementation is used to simplify the creation of the Fractal
     * architecture by loading an ADL file instead coding all,</li>
     * <li>the implementation of the sub-component 'Registry' is retrieve from:
     * <ol>
     * <li>the configured property,</li>
     * <li>the classloader,</li>
     * <li>and, loaded in the composite 'System'</li>
     * <li>the server interface
     * {@value PetalsCompositeController#SYSTEM_REGISTRY_FRACTAL_SRV_ITF_NAME}
     * of the composite is created,</li>
     * <li>all interfaces are bound.</li>
     * </ol>
     * </p>
     * 
     * @param petalsComposite
     *            The composite {@value FractalHelper#PETALS_COMPOSITE} in which
     *            the composite or component will be created.
     * @return The composite or component created
     * @throws CreateComponentException
     *             An error occurs during the instantiation of the composite
     * @throws NoRegistryImplException
     *             No registry implementation found
     * @throws TooManyRegistryImplException
     *             Too many registry implementations found
     * @throws BindingInterfaceException
     *             An error occurs during the binding of the interfaces
     */
    private final Component createAndAddCompositeSystem()
            throws CreateComponentException, NoRegistryImplException, TooManyRegistryImplException,
            BindingInterfaceException {
        this.log.start();

        // Create the base composite 'System'
        final Component systemComposite = this.createAndAddComponent(FractalHelper.SYSTEM_COMPOSITE,
                FractalHelper.SYSTEM_COMPOSITE, this.petalsComposite);

        // Remove the dummy implementation of the registry. Next it will be
        // replaced by the expected one.
        final BindingController systemCompositeBC;
        try {
            systemCompositeBC = Fractal.getBindingController(systemComposite);
            systemCompositeBC
                    .unbindFc(PetalsCompositeController.SYSTEM_REGISTRY_FRACTAL_INT_SRV_ITF_NAME);

            final ContentController systemCompositeCC = Fractal
                    .getContentController(systemComposite);
            for (final Component systemCompositeSubComponent : systemCompositeCC
                    .getFcSubComponents()) {
                final String systemCompositeSubComponentName = Fractal.getNameController(
                        systemCompositeSubComponent).getFcName();
                if (systemCompositeSubComponentName.equals(FractalHelper.REGISTRY_COMPONENT)) {
                    systemCompositeCC.removeFcSubComponent(systemCompositeSubComponent);
                    break;
                }
            }
        } catch (final NoSuchInterfaceException e) {
            throw new CreateComponentException(FractalHelper.SYSTEM_COMPOSITE, e);
        } catch (final IllegalBindingException e) {
            throw new CreateComponentException(FractalHelper.SYSTEM_COMPOSITE, e);
        } catch (final IllegalLifeCycleException e) {
            throw new CreateComponentException(FractalHelper.SYSTEM_COMPOSITE, e);
        } catch (final IllegalContentException e) {
            throw new CreateComponentException(FractalHelper.SYSTEM_COMPOSITE, e);
        }
        
        // Retrieve the registry implementation from configuration and looking up the classloader
        final String registryImplProperty = this.configurationService.getSubDomainConfiguration()
                .getRegistryImplementation();
        if (registryImplProperty != null) {
            this.log.info("Registry Implementation property found: " + registryImplProperty);
            try {
                this.registryImplDef = getClass().getClassLoader().loadClass(registryImplProperty);
            } catch (ClassNotFoundException e) {
                // TODO error instead... ?
                this.log.warning(registryImplProperty + " is not a valid class name");
            }
        }

        if (registryImplDef == null) {
            final ServiceLoader<EndpointRegistry> serviceLoader = ServiceLoader.load(EndpointRegistry.class);
            final Iterator<EndpointRegistry> servicesIterator = serviceLoader.iterator();
            if (servicesIterator.hasNext()) {
                this.registryImplDef = servicesIterator.next().getClass();
                if (servicesIterator.hasNext()) {
                    this.log.error("Too many registry implementations found.");
                    throw new TooManyRegistryImplException();
                }
            } else {
                this.log.error("No registry implementation found.");
                throw new NoRegistryImplException();
            }
        }

        this.log.info("Registry implementation to used: " + registryImplDef.getName());

        // Instantiates the registry implementation as a Fractal component
        try {
            final Component registryImpl = this.createAndAddComponent(registryImplDef,
                    FractalHelper.REGISTRY_COMPONENT, systemComposite);

            this.log.debug("Registry: Fractal component created.");

            // Next we bind client interfaces of the registry to the
            // right client interfaces of the composite 'System'
            final ContentController systemCompositeCC = Fractal.getContentController(systemComposite);

            final BindingController registryImplBC = Fractal.getBindingController(registryImpl);

            // We bind the client interface 'configuration' of the
            // component 'RegistryImpl' with the client interface
            // 'configuration' of the composite 'System'
            registryImplBC.bindFc(EndpointRegistry.CONFIGURATION_FRACTAL_ITF_NAME, systemCompositeCC
                    .getFcInternalInterface(PetalsCompositeController.SYSTEM_CONFIGURATION_FRACTAL_INT_ITF_NAME));

            // We bind the server interface 'service' of the component
            // 'RegistryImpl' with the server interface 'registry' of
            // the composite 'System'
            final Interface registryImplServiceItf = (Interface) registryImpl
                    .getFcInterface(EndpointRegistry.FRACTAL_SRV_ITF_NAME);
            systemCompositeBC.bindFc(
                    PetalsCompositeController.SYSTEM_REGISTRY_FRACTAL_INT_SRV_ITF_NAME,
                    registryImplServiceItf);

            this.log.debug("Registry: interfaces bound.");

            this.log.info("Registry loaded");

        } catch (final NoSuchInterfaceException e) {
            this.log.error("Error binding interfaces of the registry implementation '" + registryImplDef.getName()
                    + "'", e);
            throw new BindingInterfaceException(registryImplDef.getName(), e);
        } catch (final IllegalLifeCycleException e) {
            this.log.error("Error binding interfaces of the registry implementation '" + registryImplDef.getName()
                    + "'", e);
            throw new BindingInterfaceException(registryImplDef.getName(), e);
        } catch (final IllegalBindingException e) {
            this.log.error("Error binding interfaces of the registry implementation '" + registryImplDef.getName()
                    + "'", e);
            throw new BindingInterfaceException(registryImplDef.getName(), e);
        }

        return systemComposite;

    }

    /**
     * Bind interfaces of the composite 'Communication'
     * 
     * @param communicationComposite
     *            The composite 'Communication'
     * @param configurationServiceItf
     *            The server interface {@value ConfigurationService#FRACTAL_SRV_ITF_NAME} offered by the component
     *            {@value FractalHelper#CONFIGURATION_COMPONENT}.
     * @param registryServiceItf
     *            The server interface {@value EndpointRegistry#FRACTAL_SRV_ITF_NAME} offered by the component
     *            {@value FractalHelper#REGISTRY_COMPONENT}.
     * @param petalsCompositeCtrlItf
     *            The server interface {@value PetalsCompositeController#FRACTAL_SRV_ITF_NAME} offered by the component
     *            {@value FractalHelper#PETALS_COMPOSITE_CTRL_COMPONENT}.
     * @throws BindingInterfaceException
     *             An error occurs during interface binding of the composite
     */
    private final void bindCompositeCommunicationItf(final Component communicationComposite,
            final Interface configurationServiceItf, final Interface registryServiceItf,
            final Interface petalsCompositeCtrlItf) throws BindingInterfaceException {
        this.log.start(FractalHelper.COMMUNICATION_COMPOSITE);
        try {
            // We bind the client interface 'configuration' of the composite
            // 'Communication' with the server interface 'service' of the
            // component 'ConfigurationServiceImpl'
            final BindingController communicationCompositeBC = Fractal
                    .getBindingController(communicationComposite);
            communicationCompositeBC.bindFc(
                    PetalsCompositeController.COMMUNICATION_CONFIGURATION_FRACTAL_ITF_NAME,
                    configurationServiceItf);
            communicationCompositeBC.bindFc(PetalsCompositeController.COMMUNICATION_REGISTRY_FRACTAL_ITF_NAME,
                    registryServiceItf);
            communicationCompositeBC.bindFc(
                    PetalsCompositeController.COMMUNICATION_PETALS_COMPOSITE_CTRL_FRACTAL_ITF_NAME,
                    petalsCompositeCtrlItf);

        } catch (final NoSuchInterfaceException e) {
            throw new BindingInterfaceException(FractalHelper.COMMUNICATION_COMPOSITE, e);
        } catch (final IllegalLifeCycleException e) {
            throw new BindingInterfaceException(FractalHelper.COMMUNICATION_COMPOSITE, e);
        } catch (final IllegalBindingException e) {
            throw new BindingInterfaceException(FractalHelper.COMMUNICATION_COMPOSITE, e);
        } finally {
            this.log.end(FractalHelper.COMMUNICATION_COMPOSITE);
        }
    }

    /**
     * Bind interfaces of the composite {@value FractalHelper#SYSTEM_COMPOSITE}
     * 
     * @param systemComposite
     *            The composite 'System'
     * @param configurationServiceItf
     *            The server interface {@value ConfigurationService#FRACTAL_SRV_ITF_NAME} offered by the component
     *            {@value FractalHelper#CONFIGURATION_COMPONENT}.
     * @throws BindingInterfaceException
     *             An error occurs during interface binding of the composite
     */
    private final void bindCompositeSystemItf(final Component systemComposite, final Interface configurationServiceItf)
            throws BindingInterfaceException {
        this.log.start(FractalHelper.SYSTEM_COMPOSITE);
        try {
            // We bind the client interface 'configuration' of the composite
            // 'System' with the server interface 'service' of the
            // component 'ConfigurationServiceImpl'
            final BindingController systemCompositeBC = Fractal
                    .getBindingController(systemComposite);
            systemCompositeBC.bindFc(
                    PetalsCompositeController.SYSTEM_CONFIGURATION_FRACTAL_ITF_NAME,
                    configurationServiceItf);

        } catch (final NoSuchInterfaceException e) {
            throw new BindingInterfaceException(FractalHelper.SYSTEM_COMPOSITE, e);
        } catch (final IllegalLifeCycleException e) {
            throw new BindingInterfaceException(FractalHelper.SYSTEM_COMPOSITE, e);
        } catch (final IllegalBindingException e) {
            throw new BindingInterfaceException(FractalHelper.SYSTEM_COMPOSITE, e);
        } finally {
            this.log.end(FractalHelper.SYSTEM_COMPOSITE);
        }
    }

    /**
     * Bind interfaces of the composite {@value FractalHelper#EXTENSIONSMANAGER_COMPOSITE}
     * 
     * @param extensionsManagerComposite
     *            The composite 'ExtensionsManager'
     * @param configurationServiceItf
     *            The server interface {@value ConfigurationService#FRACTAL_SRV_ITF_NAME} offered by the component
     *            {@value FractalHelper#CONFIGURATION_COMPONENT}.
     * @param topologyServiceItf
     *            The server interface {@value PetalsCompositeController#COMMUNICATION_JMX_FRACTAL_SRV_ITF_NAME} offered
     *            by the the composite {@value FractalHelper#COMMUNICATION_COMPOSITE}.
     * @param jmxServiceItf
     *            The server interface {@value PetalsCompositeController#COMMUNICATION_JMX_FRACTAL_SRV_ITF_NAME} offered
     *            by the the composite {@value FractalHelper#COMMUNICATION_COMPOSITE}.
     * @param systemStateServiceItf
     *            The server interface {@value PetalsCompositeController#SYSTEM_SYSTEMSTATE_FRACTAL_SRV_ITF_NAME}
     *            offered by the the composite {@value FractalHelper#SYSTEM_COMPOSITE}.
     * @param registryServiceItf
     *            The server interface {@value PetalsCompositeController#SYSTEM_REGISTRY_FRACTAL_SRV_ITF_NAME} offered
     *            by the the composite {@value FractalHelper#SYSTEM_COMPOSITE}.
     * @param deploymentServiceItf
     *            The server interface {@value PetalsCompositeController#JBIMANAGEMENT_DEPLOYMENT_FRACTAL_SRV_ITF_NAME}
     *            offered by the the composite {@value FractalHelper#JBI_MANAGEMENT_COMPOSITE}.
     * @param installationServiceItf
     *            The server interface
     *            {@value PetalsCompositeController#JBIMANAGEMENT_INSTALLATION_FRACTAL_SRV_ITF_NAME} offered by the the
     *            composite {@value FractalHelper#JBI_MANAGEMENT_COMPOSITE}.
     * @param componentInstallationServiceItf
     *            The server interface
     *            {@value PetalsCompositeController#JBIMANAGEMENT_COMPONENTINSTALLATION_FRACTAL_SRV_ITF_NAME} offered by
     *            the the composite {@value FractalHelper#JBI_MANAGEMENT_COMPOSITE}.
     * @param atomicDeploymentServiceItf
     *            The server interface
     *            {@value PetalsCompositeController#JBIMANAGEMENT_ATOMICDEPLOYMENT_FRACTAL_SRV_ITF_NAME} offered by the
     *            the composite {@value FractalHelper#JBI_MANAGEMENT_COMPOSITE}.
     * @param adminServiceItf
     *            The server interface {@value PetalsCompositeController#JBIMANAGEMENT_ADMIN_FRACTAL_SRV_ITF_NAME}
     *            offered by the the composite {@value FractalHelper#JBI_MANAGEMENT_COMPOSITE}.
     * @param containerServiceItf
     *            The server interface {@value PetalsCompositeController#JBIMANAGEMENT_ADMIN_FRACTAL_SRV_ITF_NAME}
     *            offered by the the composite {@value FractalHelper#JBI_MANAGEMENT_COMPOSITE}.
     * @param petalsAdminServiceItf
     *            The server interface {@value PetalsAdminInterface#FRACTAL_SRV_ITF_NAME} offered by the the component
     *            {@value FractalHelper#PETALSADMIN_COMPONENT} .
     * @throws BindingInterfaceException
     *             An error occurs during interface binding of the composite
     */
    private final void bindCompositePostExtensionsManagerItf(final Component extensionsManagerComposite,
            final Interface configurationServiceItf, final Interface topologyServiceItf, final Interface jmxServiceItf,
            final Interface systemStateServiceItf, final Interface registryServiceItf,
            final Interface deploymentServiceItf, final Interface installationServiceItf,
            final Interface componentInstallationServiceItf, final Interface atomicDeploymentServiceItf,
            final Interface adminServiceItf, final Interface containerServiceItf, final Interface petalsAdminServiceItf)
            throws BindingInterfaceException {
        this.log.start(FractalHelper.POST_EXTENSIONSMANAGER_COMPOSITE);
        try {
            // We bind the client interface 'configuration' of the composite
            // 'PostExtensionsManager' with the server interface 'service' of the
            // component 'ConfigurationServiceImpl'
            final BindingController extensionsManagerCompositeBC = Fractal
                    .getBindingController(extensionsManagerComposite);
            extensionsManagerCompositeBC.bindFc(
                    PetalsCompositeController.POSTEXTENSIONSMANAGER_CONFIGURATION_FRACTAL_ITF_NAME,
                    configurationServiceItf);

            // We bind the client interface 'jmx' of the composite
            // 'PostExtensionsManager' with the server interface 'jmx' of the
            // composite 'Communication'
            extensionsManagerCompositeBC.bindFc(PetalsCompositeController.POSTEXTENSIONSMANAGER_JMX_FRACTAL_ITF_NAME,
                    jmxServiceItf);

            // We bind the client interface 'topology' of the composite
            // 'PostExtensionsManager' with the server interface 'topology' of the
            // composite 'Communication'
            extensionsManagerCompositeBC.bindFc(
                    PetalsCompositeController.POSTEXTENSIONSMANAGER_TOPOLOGY_FRACTAL_ITF_NAME,
                    topologyServiceItf);

            // We bind the client interface 'systemstate' of the composite
            // 'PostExtensionsManager' with the server interface 'systemstate' of
            // the composite 'System'
            extensionsManagerCompositeBC
                    .bindFc(PetalsCompositeController.POSTEXTENSIONSMANAGER_SYSTEMSTATE_FRACTAL_ITF_NAME,
                            systemStateServiceItf);

            // We bind the client interface 'registry' of the composite
            // 'PostExtensionsManager' with the server interface 'registry' of
            // the composite 'System'
            extensionsManagerCompositeBC.bindFc(
                    PetalsCompositeController.POSTEXTENSIONSMANAGER_REGISTRY_FRACTAL_ITF_NAME,
                    registryServiceItf);

            // We bind the client interface 'deployment' of the composite
            // 'PostExtensionsManager' with the server interface 'deploymentService'
            // of the composite 'JBI-Management'
            extensionsManagerCompositeBC.bindFc(
                    PetalsCompositeController.POSTEXTENSIONSMANAGER_DEPLOYMENT_FRACTAL_ITF_NAME,
                    deploymentServiceItf);

            // We bind the client interface 'installation' of the composite
            // 'PostExtensionsManager' with the server interface
            // 'installationService' of the composite 'JBI-Management'
            extensionsManagerCompositeBC.bindFc(
                    PetalsCompositeController.POSTEXTENSIONSMANAGER_INSTALLATION_FRACTAL_ITF_NAME,
                    installationServiceItf);

            // We bind the client interface 'componentInstallation' of the
            // composite 'PostExtensionsManager' with the server interface
            // 'componentInstallationService' of the composite 'JBI-Management'
            extensionsManagerCompositeBC.bindFc(
                    PetalsCompositeController.POSTEXTENSIONSMANAGER_COMPONENTINSTALLATION_FRACTAL_ITF_NAME,
                    componentInstallationServiceItf);

            // We bind the client interface 'atomicDeployment' of the composite
            // 'PostExtensionsManager' with the server interface
            // 'atomicDeploymentService' of the composite 'JBI-Management'
            extensionsManagerCompositeBC.bindFc(
                    PetalsCompositeController.POSTEXTENSIONSMANAGER_ATOMICDEPLOYMENT_FRACTAL_ITF_NAME,
                    atomicDeploymentServiceItf);

            // We bind the client interface 'admin' of the composite
            // 'PostExtensionsManager' with the server interface
            // 'adminService' of the composite 'JBI-Management'
            extensionsManagerCompositeBC.bindFc(PetalsCompositeController.POSTEXTENSIONSMANAGER_ADMIN_FRACTAL_ITF_NAME,
                    adminServiceItf);

            // We bind the client interface 'containerService' of the composite
            // 'PostExtensionsManager' with the server interface
            // 'containerService' of the composite 'JBI-Management'
            extensionsManagerCompositeBC.bindFc(
                    PetalsCompositeController.POSTEXTENSIONSMANAGER_CONTAINERSERVICE_FRACTAL_ITF_NAME,
                    containerServiceItf);

            // We bind the client interface 'petalsAdmin' of the composite
            // 'PostExtensionsManager' with the server interface
            // 'service' of the component 'PetalsAdmin'
            extensionsManagerCompositeBC
                    .bindFc(PetalsCompositeController.POSTEXTENSIONSMANAGER_PETALSADMIN_FRACTAL_ITF_NAME,
                            petalsAdminServiceItf);

        } catch (final NoSuchInterfaceException e) {
            throw new BindingInterfaceException(FractalHelper.POST_EXTENSIONSMANAGER_COMPOSITE, e);
        } catch (final IllegalLifeCycleException e) {
            throw new BindingInterfaceException(FractalHelper.POST_EXTENSIONSMANAGER_COMPOSITE, e);
        } catch (final IllegalBindingException e) {
            throw new BindingInterfaceException(FractalHelper.POST_EXTENSIONSMANAGER_COMPOSITE, e);
        } finally {
            this.log.end(FractalHelper.POST_EXTENSIONSMANAGER_COMPOSITE);
        }
    }

    /**
     * Bind interfaces of the composite {@value FractalHelper#JBI_MANAGEMENT_COMPOSITE}
     * 
     * @param jbiManagementComposite
     *            The composite {@value FractalHelper#JBI_MANAGEMENT_COMPOSITE}
     * @param configurationServiceItf
     *            The server interface {@value ConfigurationService#FRACTAL_SRV_ITF_NAME} offered by the component
     *            {@value FractalHelper#CONFIGURATION_COMPONENT}.
     * @param jmxServiceItf
     *            The server interface {@value PetalsCompositeController#COMMUNICATION_JMX_FRACTAL_SRV_ITF_NAME} offered
     *            by the the composite {@value FractalHelper#COMMUNICATION_COMPOSITE}.
     * @param systemStateServiceItf
     *            The server interface {@value PetalsCompositeController#SYSTEM_SYSTEMSTATE_FRACTAL_SRV_ITF_NAME}
     *            offered by the the composite {@value FractalHelper#SYSTEM_COMPOSITE}.
     * @param classloaderServiceItf
     *            The server interface {@value PetalsCompositeController#SYSTEM_CLASSLOADER_FRACTAL_SRV_ITF_NAME}
     *            offered by the the composite {@value FractalHelper#SYSTEM_COMPOSITE}.
     * @param repositoryServiceItf
     *            The server interface {@value PetalsCompositeController#SYSTEM_REPOSITORY_FRACTAL_SRV_ITF_NAME} offered
     *            by the the composite {@value FractalHelper#SYSTEM_COMPOSITE}.
     * @param registryServiceItf
     *            The server interface {@value PetalsCompositeController#SYSTEM_REGISTRY_FRACTAL_SRV_ITF_NAME} offered
     *            by the the composite {@value FractalHelper#SYSTEM_COMPOSITE}.
     * @param routerServiceItf
     *            The server interface {@value PetalsCompositeController#JBIMANAGEMENT_ROUTER_FRACTAL_ITF_NAME} offered
     *            by the the composite {@value FractalHelper#JBI_MESSAGING_COMPOSITE}.
     * @param installationExtensionsManagerItf
     *            The server interface
     *            {@value PetalsCompositeController#JBIMANAGEMENT_INSTALLATIONEXTENSIONSMANAGER_FRACTAL_ITF_NAME}
     *            offered by the the composite {@value FractalHelper#EXTENSIONSMANAGER_COMPOSITE}.
     * @param jndiServiceItf
     *            The server interface {@value PetalsCompositeController#COMMUNICATION_JNDI_FRACTAL_SRV_ITF_NAME}
     *            offered by the the composite {@value FractalHelper#COMMUNICATION_COMPOSITE}.
     * @throws BindingInterfaceException
     *             An error occurs during interface binding of the composite
     */
    private final void bindCompositeJbiManagementItf(final Component jbiManagementComposite,
            final Interface configurationServiceItf, final Interface jmxServiceItf,
            final Interface systemStateServiceItf, final Interface classloaderServiceItf,
            final Interface repositoryServiceItf, final Interface registryServiceItf, final Interface routerServiceItf,
            final Interface installationExtensionsManagerItf,
            final Interface jndiServiceItf) throws BindingInterfaceException {
        this.log.start(FractalHelper.JBI_MANAGEMENT_COMPOSITE);
        try {
            // We bind the client interface 'configuration' of the composite
            // 'JBI-Management' with the server interface 'service' of the
            // component 'ConfigurationServiceImpl'
            final BindingController jbiManagementCompositeBC = Fractal
                    .getBindingController(jbiManagementComposite);
            jbiManagementCompositeBC.bindFc(
                    PetalsCompositeController.JBIMANAGEMENT_CONFIGURATION_FRACTAL_ITF_NAME,
                    configurationServiceItf);

            // We bind the client interface 'jmx' of the composite
            // 'JBI-Management' with the server interface 'jmx' of the
            // composite 'Communication'
            jbiManagementCompositeBC.bindFc(
                    PetalsCompositeController.JBIMANAGEMENT_JMX_FRACTAL_ITF_NAME, jmxServiceItf);

            // We bind the client interface 'systemstate' of the composite
            // 'JBI-Management' with the server interface 'systemstate' of
            // the composite 'System'
            jbiManagementCompositeBC.bindFc(
                    PetalsCompositeController.JBIMANAGEMENT_SYSTEMSTATE_FRACTAL_ITF_NAME,
                    systemStateServiceItf);

            // We bind the client interface 'classloader' of the composite
            // 'JBI-Management' with the server interface 'classloader' of the
            // composite 'System'
            jbiManagementCompositeBC.bindFc(
                    PetalsCompositeController.JBIMANAGEMENT_CLASSLOADER_FRACTAL_ITF_NAME,
                    classloaderServiceItf);

            // We bind the client interface 'repository' of the composite
            // 'JBI-Management' with the server interface 'repository' of
            // the composite 'System'
            jbiManagementCompositeBC.bindFc(
                    PetalsCompositeController.JBIMANAGEMENT_REPOSITORY_FRACTAL_ITF_NAME,
                    repositoryServiceItf);

            // We bind the client interface 'endpoint' of the composite
            // 'JBI-Management' with the server interface 'registry' of the
            // composite 'System'
            jbiManagementCompositeBC.bindFc(
                    PetalsCompositeController.JBIMANAGEMENT_REGISTRY_FRACTAL_ITF_NAME,
                    registryServiceItf);

            // We bind the client interface 'router' of the composite
            // 'JBI-Management' with the server interface 'router' of the
            // composite 'JBI-Messaging'
            jbiManagementCompositeBC.bindFc(
                    PetalsCompositeController.JBIMANAGEMENT_ROUTER_FRACTAL_ITF_NAME,
                    routerServiceItf);

            // We bind the client interface 'installationExtensionManager' of
            // the composite 'JBI-Management' with the server interface
            // 'installationExtensionManager' of the composite
            // 'ExtensionsManager'
            jbiManagementCompositeBC
                    .bindFc(PetalsCompositeController.JBIMANAGEMENT_INSTALLATIONEXTENSIONSMANAGER_FRACTAL_ITF_NAME,
                            installationExtensionsManagerItf);

            // We bind the client interface 'jndi' of
            // the composite 'JBI-Management' with the server interface
            // 'jndi' of the composite
            // 'Communication'
            jbiManagementCompositeBC.bindFc(PetalsCompositeController.JBIMANAGEMENT_JNDI_FRACTAL_ITF_NAME,
                    jndiServiceItf);

        } catch (final NoSuchInterfaceException e) {
            throw new BindingInterfaceException(FractalHelper.JBI_MANAGEMENT_COMPOSITE, e);
        } catch (final IllegalLifeCycleException e) {
            throw new BindingInterfaceException(FractalHelper.JBI_MANAGEMENT_COMPOSITE, e);
        } catch (final IllegalBindingException e) {
            throw new BindingInterfaceException(FractalHelper.JBI_MANAGEMENT_COMPOSITE, e);
        } finally {
            this.log.end(FractalHelper.JBI_MANAGEMENT_COMPOSITE);
        }
    }

    /**
     * Bind interfaces of the composite {@value FractalHelper#PETALSADMIN_COMPONENT}
     * 
     * @param petalsAdminServiceComponent
     *            The composite {@value FractalHelper#JBI_MANAGEMENT_COMPOSITE}
     * @param configurationServiceItf
     *            The server interface {@value ConfigurationService#FRACTAL_SRV_ITF_NAME} offered by the component
     *            {@value FractalHelper#CONFIGURATION_COMPONENT}.
     * @param topologyServiceItf
     *            The server interface {@value PetalsCompositeController#COMMUNICATION_TOPOLOGY_FRACTAL_SRV_ITF_NAME}
     *            offered by the the composite {@value FractalHelper#SYSTEM_COMPOSITE}.
     * @param deploymentServiceItf
     *            The server interface {@value PetalsCompositeController#JBIMANAGEMENT_DEPLOYMENT_FRACTAL_SRV_ITF_NAME}
     *            offered by the the composite {@value FractalHelper#JBI_MANAGEMENT_COMPOSITE}.
     * @param installationServiceItf
     *            The server interface
     *            {@value PetalsCompositeController#JBIMANAGEMENT_INSTALLATION_FRACTAL_SRV_ITF_NAME} offered by the the
     *            composite {@value FractalHelper#JBI_MANAGEMENT_COMPOSITE}.
     * @param registryServiceItf
     *            The server interface {@value PetalsCompositeController#SYSTEM_REGISTRY_FRACTAL_SRV_ITF_NAME} offered
     *            by the the composite {@value FractalHelper#SYSTEM_COMPOSITE}.
     * @throws BindingInterfaceException
     *             An error occurs during interface binding of the composite
     */
    private final void bindComponentPetalsAdminService(final Component petalsAdminServiceComponent,
            final Interface configurationServiceItf, final Interface topologyServiceItf,
            final Interface deploymentServiceItf, final Interface installationServiceItf,
            final Interface registryServiceItf) throws BindingInterfaceException {
        this.log.start(FractalHelper.PETALSADMIN_COMPONENT);
        try {
            // We bind the client interface 'configuration' of the component
            // 'PetalsAdminServiceImpl' with the server interface 'service' of
            // the component 'ConfigurationServiceImpl'
            final BindingController petalsAdminServiceComponentBC = Fractal
                    .getBindingController(petalsAdminServiceComponent);
            petalsAdminServiceComponentBC.bindFc(
                    PetalsCompositeController.PETALSADMIN_CONFIGURATION_FRACTAL_ITF_NAME,
                    configurationServiceItf);

            // We bind the client interface 'topology' of the composite
            // 'PetalsAdminServiceImpl' with the server interface 'topology' of
            // the composite 'System'
            petalsAdminServiceComponentBC.bindFc(
                    PetalsCompositeController.PETALSADMIN_TOPOLOGY_FRACTAL_ITF_NAME,
                    topologyServiceItf);

            // We bind the client interface 'deployment' of the composite
            // 'PetalsAdminServiceImpl' with the server interface
            // 'deploymentService' of the composite 'JBI-Management'
            petalsAdminServiceComponentBC.bindFc(
                    PetalsCompositeController.PETALSADMIN_DEPLOYMENT_FRACTAL_ITF_NAME,
                    deploymentServiceItf);

            // We bind the client interface 'installation' of the composite
            // 'PetalsAdminServiceImpl' with the server interface
            // 'installationService' of the composite 'JBI-Management'
            petalsAdminServiceComponentBC.bindFc(
                    PetalsCompositeController.PETALSADMIN_INSTALLATION_FRACTAL_ITF_NAME,
                    installationServiceItf);

            // We bind the client interface 'endpoint' of the composite
            // 'PetalsAdminServiceImpl' with the server interface 'registry' of
            // the composite 'System'
            petalsAdminServiceComponentBC.bindFc(
                    PetalsCompositeController.PETALSADMIN_REGISTRY_FRACTAL_ITF_NAME,
                    registryServiceItf);

        } catch (final NoSuchInterfaceException e) {
            throw new BindingInterfaceException(FractalHelper.PETALSADMIN_COMPONENT, e);
        } catch (final IllegalLifeCycleException e) {
            throw new BindingInterfaceException(FractalHelper.PETALSADMIN_COMPONENT, e);
        } catch (final IllegalBindingException e) {
            throw new BindingInterfaceException(FractalHelper.PETALSADMIN_COMPONENT, e);
        } finally {
            this.log.end(FractalHelper.PETALSADMIN_COMPONENT);
        }
    }

    /**
     * Bind interfaces of the composite {@value FractalHelper#JBI_MESSAGING_COMPOSITE}
     * 
     * @param jbiManagementComposite
     *            The composite {@value FractalHelper#JBI_MANAGEMENT_COMPOSITE}
     * @param configurationServiceItf
     *            The server interface {@value ConfigurationService#FRACTAL_SRV_ITF_NAME} offered by the component
     *            {@value FractalHelper#CONFIGURATION_COMPONENT}.
     * @param topologyServiceItf
     *            The server interface {@value PetalsCompositeController#COMMUNICATION_TOPOLOGY_FRACTAL_SRV_ITF_NAME}
     *            offered by the the composite {@value FractalHelper#COMMUNICATION_COMPOSITE}.
     * @param jndiServiceItf
     *            The server interface {@value PetalsCompositeController#COMMUNICATION_JNDI_FRACTAL_SRV_ITF_NAME}
     *            offered by the the composite {@value FractalHelper#COMMUNICATION_COMPOSITE}.
     * @param jmxServiceItf
     *            The server interface {@value PetalsCompositeController#COMMUNICATION_JMX_FRACTAL_SRV_ITF_NAME} offered
     *            by the the composite {@value FractalHelper#COMMUNICATION_COMPOSITE}.
     * @param persistenceServiceItf
     *            The server interface {@value PetalsCompositeController#SYSTEM_PERSISTENCE_FRACTAL_SRV_ITF_NAME}
     *            offered by the the composite {@value FractalHelper#SYSTEM_COMPOSITE}.
     * @param transporterLocalServiceItf
     *            The server interface {@value PetalsCompositeController#TRANSPORTER_LOCAL_FRACTAL_SRV_ITF_NAME} offered
     *            by the the composite {@value FractalHelper#TRANSPORTER_COMPOSITE}.
     * @param transporterTcpServiceItf
     *            The server interface {@value PetalsCompositeController#TRANSPORTER_TCP_FRACTAL_SRV_ITF_NAME} offered
     *            by the the composite {@value FractalHelper#TRANSPORTER_COMPOSITE}.
     * @param registryServiceItf
     *            The server interface {@value PetalsCompositeController#SYSTEM_REGISTRY_FRACTAL_SRV_ITF_NAME} offered
     *            by the the composite {@value FractalHelper#SYSTEM_COMPOSITE}.
     * @throws BindingInterfaceException
     *             An error occurs during interface binding of the composite
     */
    private final void bindCompositeJbiMessagingItf(final Component jbiManagementComposite,
            final Interface configurationServiceItf, final Interface topologyServiceItf,
            final Interface jndiServiceItf, final Interface jmxServiceItf, final Interface persistenceServiceItf,
            final Interface transporterLocalServiceItf, final Interface transporterTcpServiceItf,
            final Interface registryServiceItf) throws BindingInterfaceException {
        this.log.start(FractalHelper.JBI_MESSAGING_COMPOSITE);
        try {
            // We bind the client interface 'configuration' of the composite
            // 'JBI-Messaging' with the server interface 'service' of the
            // component 'ConfigurationServiceImpl'
            final BindingController jbiMessagingCompositeBC = Fractal
                    .getBindingController(jbiManagementComposite);
            jbiMessagingCompositeBC.bindFc(
                    PetalsCompositeController.JBIMESSAGING_CONFIGURATION_FRACTAL_ITF_NAME,
                    configurationServiceItf);

            // We bind the client interface 'topology' of the composite
            // 'JBI-Messaging' with the server interface 'topology' of the
            // composite 'System'
            jbiMessagingCompositeBC.bindFc(
                    PetalsCompositeController.JBIMESSAGING_TOPOLOGY_FRACTAL_ITF_NAME,
                    topologyServiceItf);

            // We bind the client interface 'jndi' of the composite
            // 'JBI-Messaging' with the server interface 'jndi' of the
            // composite 'Communication'
            jbiMessagingCompositeBC.bindFc(
                    PetalsCompositeController.JBIMESSAGING_JNDI_FRACTAL_ITF_NAME, jndiServiceItf);

            // We bind the client interface 'jmx' of the composite
            // 'JBI-Messaging' with the server interface 'jmx' of the
            // composite 'Communication'
            jbiMessagingCompositeBC.bindFc(
                    PetalsCompositeController.JBIMESSAGING_JMX_FRACTAL_ITF_NAME, jmxServiceItf);

            // We bind the client interface 'persistence' of the composite
            // 'JBI-Messaging' with the server interface 'persistence' of
            // the composite 'System'
            jbiMessagingCompositeBC.bindFc(
                    PetalsCompositeController.JBIMESSAGING_PERSISTENCE_FRACTAL_ITF_NAME,
                    persistenceServiceItf);

            // We bind the client interface 'transporter-local' of the composite
            // 'JBI-Messaging' with the server interface 'transporter-local' of
            // the composite 'Transporter'
            jbiMessagingCompositeBC.bindFc(
                    PetalsCompositeController.JBIMESSAGING_TRANSPORTERLOCAL_FRACTAL_ITF_NAME,
                    transporterLocalServiceItf);

            // We bind the client interface 'transporter-tcp' of the composite
            // 'JBI-Messaging' with the server interface 'transporter-tcp' of
            // the composite 'Transporter'
            jbiMessagingCompositeBC.bindFc(
                    PetalsCompositeController.JBIMESSAGING_TRANSPORTERTCP_FRACTAL_ITF_NAME,
                    transporterTcpServiceItf);

            // We bind the client interface 'endpoint' of the composite
            // 'JBI-Messaging' with the server interface 'registry' of the
            // composite 'System'
            jbiMessagingCompositeBC.bindFc(
                    PetalsCompositeController.JBIMESSAGING_REGISTRY_FRACTAL_ITF_NAME,
                    registryServiceItf);

        } catch (final NoSuchInterfaceException e) {
            throw new BindingInterfaceException(FractalHelper.JBI_MESSAGING_COMPOSITE, e);
        } catch (final IllegalLifeCycleException e) {
            throw new BindingInterfaceException(FractalHelper.JBI_MESSAGING_COMPOSITE, e);
        } catch (final IllegalBindingException e) {
            throw new BindingInterfaceException(FractalHelper.JBI_MESSAGING_COMPOSITE, e);
        } finally {
            this.log.end(FractalHelper.JBI_MESSAGING_COMPOSITE);
        }
    }

    /**
     * Bind interfaces of the composite {@value FractalHelper#TRANSPORTER_COMPOSITE}
     * 
     * @param transporterComposite
     *            The composite 'System'
     * @param configurationServiceItf
     *            The server interface {@value ConfigurationService#FRACTAL_SRV_ITF_NAME} offered by the component
     *            {@value FractalHelper#CONFIGURATION_COMPONENT}.
     * @param topologyServiceItf
     *            The server interface {@value PetalsCompositeController#COMMUNICATION_JMX_FRACTAL_SRV_ITF_NAME} offered
     *            by the the composite {@value FractalHelper#COMMUNICATION_COMPOSITE}.
     * @param transportListenerServiceItf
     *            The server interface
     *            {@value PetalsCompositeController#JBIMESSAGING_TRANSPORTERLISTENER_FRACTAL_SRV_ITF_NAME} offered by
     *            the the composite {@value FractalHelper#JBI_MESSAGING_COMPOSITE}.
     * @throws BindingInterfaceException
     *             An error occurs during interface binding of the composite
     */
    private final void bindCompositeTransporterItf(final Component transporterComposite,
            final Interface configurationServiceItf, final Interface topologyServiceItf,
            final Interface transportListenerServiceItf) throws BindingInterfaceException {
        this.log.start(FractalHelper.TRANSPORTER_COMPOSITE);
        try {
            // We bind the client interface 'configuration' of the composite
            // 'Transporter' with the server interface 'service' of the
            // component 'ConfigurationServiceImpl'
            final BindingController systemCompositeBC = Fractal
                    .getBindingController(transporterComposite);
            systemCompositeBC.bindFc(
                    PetalsCompositeController.TRANSPORTER_CONFIGURATION_FRACTAL_ITF_NAME,
                    configurationServiceItf);

            // We bind the client interface 'topology' of the composite
            // 'Transporter' with the server interface 'topology' of the
            // composite 'Communication'
            systemCompositeBC.bindFc(
                    PetalsCompositeController.TRANSPORTER_TOPOLOGY_FRACTAL_ITF_NAME,
                    topologyServiceItf);

            // We bind the client interface 'transportlistener' of the composite
            // 'Transporter' with the server interface 'transportlistener' of
            // the composite 'JBI-Messaging'
            systemCompositeBC.bindFc(
                    PetalsCompositeController.TRANSPORTER_TRANSPORTLISTENER_FRACTAL_ITF_NAME,
                    transportListenerServiceItf);

        } catch (final NoSuchInterfaceException e) {
            throw new BindingInterfaceException(FractalHelper.TRANSPORTER_COMPOSITE, e);
        } catch (final IllegalLifeCycleException e) {
            throw new BindingInterfaceException(FractalHelper.TRANSPORTER_COMPOSITE, e);
        } catch (final IllegalBindingException e) {
            throw new BindingInterfaceException(FractalHelper.TRANSPORTER_COMPOSITE, e);
        } finally {
            this.log.end(FractalHelper.TRANSPORTER_COMPOSITE);
        }
    }

    /**
     * Stops the registry manager.
     */
    @Lifecycle(step = Step.STOP)
    public void stop() {
        this.log.call();
    }

    /**
     * {@inheritDoc}
     */
    public void stopJbiArtefacts() throws PetalsCompositeCtrlException {
        this.log.start();
        try {
            final Component containerCtrlComponent = FractalHelper.getRecursiveComponentByName(this.petalsCompositeCC,
                    FractalHelper.CONTAINER_CONTROLLER_COMPONENT);
            if (containerCtrlComponent != null) {

                final ContainerController containerController = (ContainerController) containerCtrlComponent
                        .getFcInterface(ContainerController.FRACTAL_SRV_ITF_NAME);

                containerController.stopAllJbiArtefacts();

            } else {
                throw new PetalsCompositeCtrlException("Fractal architecture invalid: the component '"
                        + FractalHelper.CONTAINER_CONTROLLER_COMPONENT + "' is not found.");
            }
        } catch (final NoSuchInterfaceException e) {
            throw new PetalsCompositeCtrlException(e);
        } catch (final ContainerCtrlException e) {
            throw new PetalsCompositeCtrlException(e);
        } finally {
            this.log.end();
        }
    }

    private void startJbiArtefacts() throws PetalsCompositeCtrlException {
        this.log.start();
        try {
            final Component containerCtrlComponent = FractalHelper.getRecursiveComponentByName(this.petalsCompositeCC,
                    FractalHelper.CONTAINER_CONTROLLER_COMPONENT);
            if (containerCtrlComponent != null) {

                final ContainerController containerController = (ContainerController) containerCtrlComponent
                        .getFcInterface(ContainerController.FRACTAL_SRV_ITF_NAME);

                containerController.startAllJbiArtefacts();

            } else {
                throw new PetalsCompositeCtrlException("Fractal architecture invalid: the component '"
                        + FractalHelper.CONTAINER_CONTROLLER_COMPONENT + "' is not found.");
            }
        } catch (final NoSuchInterfaceException e) {
            throw new PetalsCompositeCtrlException(e);
        } catch (final ContainerCtrlException e) {
            throw new PetalsCompositeCtrlException(e);
        } finally {
            this.log.end();
        }
    }

    /**
     * {@inheritDoc}
     */
    public void freeAllJbiArtifacts() throws PetalsCompositeCtrlException {

        this.log.start();
        try {
            final Component containerServiceComponent = FractalHelper.getRecursiveComponentByName(
                    this.petalsCompositeCC, FractalHelper.CONTAINER_SERVICE_COMPONENT);
            if (containerServiceComponent != null) {

                final Component systemStateComponent = FractalHelper.getRecursiveComponentByName(
                        this.petalsCompositeCC, FractalHelper.SYSTEMSTATE_COMPONENT);
                if (systemStateComponent != null) {

                    final ContainerService containerService = (ContainerService) containerServiceComponent
                            .getFcInterface(ContainerService.FRACTAL_SRV_ITF_NAME);

                    this.log.debug("Freeing service assembly life cycles ...");
                    for (final String saName : containerService.getServiceAssemblies().keySet()) {
                        try {
                            containerService.removeServiceAssemblyLifeCycle(saName);
                        } catch (final PetalsException e) {
                            this.log.warning("An error occurs removing the service assembly life cycle '" + saName
                                    + "'", e);
                        }
                    }
                    this.log.debug("Service assembly life cycles freed.");

                    this.log.debug("Freeing component life cycles ...");
                    for (final String name : containerService.getComponentLifecycles().keySet()) {
                        try {
                            containerService.unregisterComponentLifeCycleMBean(name);
                        } catch (final ComponentLifecycleNotFoundException e) {
                            this.log.warning(e.getMessage());
                        } catch (final ContainerServiceException e) {
                            this.log.warning("An error occurs unregistering the component life cycle MBean '" + name
                                    + "'", e);
                        }

                        try {
                            containerService.removeComponentLifeCycle(name);
                        } catch (final PetalsException e) {
                            this.log.warning("An error occurs removing the component life cycle '" + name + "'", e);
                        }
                    }
                    this.log.debug("Component life cycles freed.");

                    this.log.debug("Freeing component installers ...");
                    for (final String installerName : containerService.getInstallers().keySet()) {
                        try {
                            containerService.unregisterInstallerMBean(installerName);
                        } catch (final InstallerNotFoundException e) {
                            this.log.warning(e.getMessage());
                        } catch (final ContainerServiceException e) {
                            this.log.warning("An error occurs unregistering the component installer MBean '"
                                    + installerName + "'", e);
                        }

                        try {
                            containerService.removeInstaller(installerName);
                        } catch (final PetalsException e) {
                            this.log.warning(
                                    "An error occurs removing the component installer '" + installerName + "'", e);
                        }
                    }
                    this.log.debug("Component installers freed.");

                    this.log.debug("Freeing shared library life cycles ...");
                    for (final SharedLibraryBean slBean : containerService.getSharedLibraries().keySet()) {
                        final String slName = slBean.getName();
                        final String slVersion = slBean.getVersion();
                        try {
                            containerService.removeSharedLibraryLifeCycle(slName, slVersion);
                        } catch (final PetalsException e) {
                            this.log.warning("An error occurs removing the shared library life cycle '" + slName
                                    + "', '" + slVersion + "'", e);
                        }
                    }
                    this.log.debug("Shared library life cycles freed.");
                } else {
                    throw new PetalsCompositeCtrlException("Fractal architecture invalid: the component '"
                            + FractalHelper.SYSTEMSTATE_COMPONENT + "' is not found.");
                }
            } else {
                throw new PetalsCompositeCtrlException("Fractal architecture invalid: the component '"
                        + FractalHelper.CONTAINER_CONTROLLER_COMPONENT + "' is not found.");
            }
        } catch (final NoSuchInterfaceException e) {
            throw new PetalsCompositeCtrlException(e);
        } finally {
            this.log.end();
        }
    }

    /**
     * {@inheritDoc}
     * 
     * <p>
     * It consists in stopping the Fractal composite 'JBI-Messaging' followed by the composite 'Transporter'.
     * </p>
     */
    public void stopMessagingSubsystem() throws PetalsCompositeCtrlException {
        this.log.info("Stopping messaging sub-system ...");
        try {

            this.stopJbiArtefacts();

            final Component jbiMessagingComposite = FractalHelper.getRecursiveComponentByName(this.petalsCompositeCC,
                    FractalHelper.JBI_MESSAGING_COMPOSITE);
            this.log.debug("Stops composite " + FractalHelper.JBI_MESSAGING_COMPOSITE);
            FractalHelper.stopComponent(jbiMessagingComposite);

            final Component transporterComposite = FractalHelper.getRecursiveComponentByName(petalsCompositeCC,
                    FractalHelper.TRANSPORTER_COMPOSITE);
            this.log.debug("Stops composite " + FractalHelper.TRANSPORTER_COMPOSITE);
            FractalHelper.stopComponent(transporterComposite);

            this.log.info("Messaging sub-system stopped");
        } catch (final NoSuchInterfaceException e) {
            throw new PetalsCompositeCtrlException("An error occurs stopping the messaging sub-system", e);
        } catch (final IllegalLifeCycleException e) {
            throw new PetalsCompositeCtrlException("An error occurs stopping the messaging sub-system", e);
        }
    }

    /**
     * {@inheritDoc}
     * 
     * <p>
     * It consists in starting the Fractal composite 'Transporter' followed by the composite 'JBI-Messaging'.
     * </p>
     */
    public void startMessagingSubsystem() throws PetalsCompositeCtrlException {
        this.log.info("Starting messaging sub-system ...");
        try {
            final Component transporterComposite = FractalHelper.getRecursiveComponentByName(this.petalsCompositeCC,
                    FractalHelper.TRANSPORTER_COMPOSITE);
            this.log.debug("Starts composite " + FractalHelper.TRANSPORTER_COMPOSITE);
            FractalHelper.startComponent(transporterComposite);

            final Component jbiMessagingComposite = FractalHelper.getRecursiveComponentByName(petalsCompositeCC,
                    FractalHelper.JBI_MESSAGING_COMPOSITE);
            this.log.debug("Starts composite " + FractalHelper.JBI_MESSAGING_COMPOSITE);
            FractalHelper.startComponent(jbiMessagingComposite);

            this.startJbiArtefacts();

            this.log.info("Messaging sub-system started");
        } catch (final NoSuchInterfaceException e) {
            throw new PetalsCompositeCtrlException("An error occurs starting the messaging sub-system", e);
        } catch (final IllegalLifeCycleException e) {
            throw new PetalsCompositeCtrlException("An error occurs starting the messaging sub-system", e);
        }
    }

    /**
     * {@inheritDoc}
     */
    public void stopRegistryComponent() throws PetalsCompositeCtrlException {
        this.log.debug("Stopping the registry ...");
        try {
            final Component registryComponent = FractalHelper.getRecursiveComponentByName(this.petalsCompositeCC,
                    FractalHelper.REGISTRY_COMPONENT);
            FractalHelper.stopComponent(registryComponent);

            this.log.debug("Registry stopped");
        } catch (final NoSuchInterfaceException e) {
            throw new PetalsCompositeCtrlException("An error occurs stopping the registry", e);
        } catch (final IllegalLifeCycleException e) {
            throw new PetalsCompositeCtrlException("An error occurs stopping the registry", e);
        }
    }

    /**
     * {@inheritDoc}
     */
    public void startRegistryComponent() throws PetalsCompositeCtrlException {
        this.log.debug("Starting the registry ...");
        try {
            final Component registryComponent = FractalHelper.getRecursiveComponentByName(this.petalsCompositeCC,
                    FractalHelper.REGISTRY_COMPONENT);
            FractalHelper.startComponent(registryComponent);

            this.log.debug("Registry started");
        } catch (final NoSuchInterfaceException e) {
            throw new PetalsCompositeCtrlException("An error occurs starting the registry", e);
        } catch (final IllegalLifeCycleException e) {
            throw new PetalsCompositeCtrlException("An error occurs starting the registry", e);
        }
    }

    /**
     * {@inheritDoc}
     */
    public String getRegistryImplementation() {
        return this.registryImplDef.getName();
    }

    /**
     * {@inheritDoc}
     */
    public Properties getRegistryConfiguration(final String providedTopologyPassphrase)
            throws PetalsCompositeCtrlException {
        try {
            final Component registryComponent = FractalHelper.getRecursiveComponentByName(this.petalsCompositeCC,
                    FractalHelper.REGISTRY_COMPONENT);
            final EndpointRegistry registry = (EndpointRegistry) registryComponent
                    .getFcInterface(EndpointRegistry.FRACTAL_SRV_ITF_NAME);
            return registry.getConfiguration(providedTopologyPassphrase);

        } catch (final NoSuchInterfaceException e) {
            throw new PetalsCompositeCtrlException(
                    "An error occurs getting the configuration of the registry mutable implementation", e);
        } catch (final MutableImplException e) {
            throw new PetalsCompositeCtrlException(
                    "An error occurs getting the configuration of the registry mutable implementation", e);
        }
    }

    /**
     * {@inheritDoc}
     */
    public void stopSystemRecoveryService() throws PetalsCompositeCtrlException {
        this.log.debug("Stopping the system recovery service ...");
        try {
            final Component systemRecoveryComponent = FractalHelper.getRecursiveComponentByName(this.petalsCompositeCC,
                    FractalHelper.SYSTEMRECOVERY_COMPONENT);
            FractalHelper.stopComponent(systemRecoveryComponent);

            this.log.debug("System recovery service stopped");
        } catch (final NoSuchInterfaceException e) {
            throw new PetalsCompositeCtrlException("An error occurs stopping the system recovery service", e);
        } catch (final IllegalLifeCycleException e) {
            throw new PetalsCompositeCtrlException("An error occurs stopping the system recovery service", e);
        }
    }

    /**
     * {@inheritDoc}
     */
    public void startSystemRecoveryService() throws PetalsCompositeCtrlException {
        this.log.debug("Starting the system recovery service ...");
        try {
            final Component systemRecoveryComponent = FractalHelper.getRecursiveComponentByName(this.petalsCompositeCC,
                    FractalHelper.SYSTEMRECOVERY_COMPONENT);
            FractalHelper.startComponent(systemRecoveryComponent);

            this.log.debug("System recovery service started");
        } catch (final NoSuchInterfaceException e) {
            throw new PetalsCompositeCtrlException("An error occurs starting the system recovery service", e);
        } catch (final IllegalLifeCycleException e) {
            throw new PetalsCompositeCtrlException("An error occurs starting the system recovery service", e);
        }
    }

    /**
     * {@inheritDoc}
     */
    public void recoverSystem() throws PetalsCompositeCtrlException {
        this.log.start();
        try {
            this.startSystemRecoveryService();

            final Component systemRecoveryComponent = FractalHelper.getRecursiveComponentByName(this.petalsCompositeCC,
                    FractalHelper.SYSTEMRECOVERY_COMPONENT);
            final SystemRecoveryService systemRecoveryService = (SystemRecoveryService) systemRecoveryComponent
                    .getFcInterface(SystemRecoveryService.FRACTAL_SRV_ITF_NAME);

            systemRecoveryService.recoverAllEntities();

            this.stopSystemRecoveryService();
        } catch (final ManagementException e) {
            throw new PetalsCompositeCtrlException("An error occurs starting the system recovery service", e);
        } catch (final NoSuchInterfaceException e) {
            throw new PetalsCompositeCtrlException("An error occurs starting the system recovery service", e);
        } finally {
            this.log.end();
        }
    }

    /**
     * {@inheritDoc}
     */
    public void onConfigurationUpdated() throws PetalsCompositeCtrlException {
        this.preExtensionsMgr.onConfigurationUpdated();
    }

}
