/**
 * Copyright (c) 2014-2015 Linagora
 * 
 * This program/library is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License as published by
 * the Free Software Foundation, either version 2.1 of the License, or (at your
 * option) any later version.
 * 
 * This program/library is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE. See the GNU Lesser General Public License
 * for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public License
 * along with this program/library; If not, see <http://www.gnu.org/licenses/>
 * for the GNU Lesser General Public License version 2.1.
 */
package org.ow2.petals.microkernel.communication.topology;

import java.io.IOException;
import java.lang.reflect.InvocationTargetException;
import java.util.Properties;

import org.easymock.classextension.EasyMock;
import org.junit.Rule;
import org.junit.Test;
import org.ow2.petals.basisapi.exception.PetalsException;
import org.ow2.petals.clientserverapi.configuration.ContainerConfiguration;
import org.ow2.petals.clientserverapi.jbi.messaging.registry.exception.RegistryException;
import org.ow2.petals.clientserverapi.topology.exception.AttachContainerException;
import org.ow2.petals.clientserverapi.topology.exception.InconsistentContainerNameException;
import org.ow2.petals.clientserverapi.topology.exception.InconsistentSubdomainNameException;
import org.ow2.petals.clientserverapi.topology.exception.InconsistentSubdomainRegistryImplException;
import org.ow2.petals.clientserverapi.topology.exception.NoContainerConfigurationProvidedException;
import org.ow2.petals.clientserverapi.topology.exception.NoContainerNameProvidedException;
import org.ow2.petals.clientserverapi.topology.exception.NoSecurityPassPhraseProvidedException;
import org.ow2.petals.clientserverapi.topology.exception.NoSubdomainConfigurationProvidedException;
import org.ow2.petals.clientserverapi.topology.exception.SubdomainIsNotDynamicException;
import org.ow2.petals.jmx.api.api.TopologyServiceClient;
import org.ow2.petals.jmx.api.api.exception.ConnectionErrorException;
import org.ow2.petals.jmx.api.mock.junit.PetalsJmxApiJunitRule;
import org.ow2.petals.microkernel.api.communication.topology.TopologyService;
import org.ow2.petals.microkernel.api.configuration.ConfigurationService;
import org.ow2.petals.microkernel.api.jbi.messaging.registry.EndpointRegistry;
import org.ow2.petals.microkernel.configuration.ConfigurationServiceImpl;
import org.ow2.petals.microkernel.configuration.ConfigurationTestUtils;
import org.ow2.petals.microkernel.configuration.LocalServerPropertiesBuilder;
import org.ow2.petals.microkernel.jbi.messaging.registry.DummyRegistryImpl;
import org.ow2.petals.microkernel.server.PetalsCompositeController;
import org.ow2.petals.topology.generated.Container;
import org.ow2.petals.topology.generated.Domain;
import org.ow2.petals.topology.generated.JmxService;
import org.ow2.petals.topology.generated.Subdomain;
import org.ow2.petals.topology.generated.SubdomainMode;
import org.ow2.petals.topology.generated.Topology;
import org.ow2.petals.topology.generated.TransportService;

import com.ebmwebsourcing.easycommons.lang.reflect.ReflectionHelper;

/**
 * Unit tests of {@link TopologyServiceImpl#attachContainerTo(ContainerConfiguration, String)}
 * 
 * @author Christophe DENEUX - Linagora
 * 
 */
public class TopologyServiceImpl_AttachContainerToTest extends AbstractTopologyServiceImplTest {

    private final static String TARGET_DOMAIN_NAME = "target-domain-name";

    private final static String TARGET_SUBDOMAIN_NAME = "target-sub-domain-name";

    private final static String TARGET_CONTAINER_NAME = "target-container-name";

    /**
     * Port of the JMX server of the target container
     */
    private final static Integer TARGET_JMX_PORT = Integer.valueOf(7070);

    /**
     * Hostname of the target container
     */
    private final static String TARGET_JMX_HOST = "localhost";

    private final static String TARGET_JMX_USER = "petals-user";

    private final static String TARGET_JMX_PWD = "petals-pwd";

    private final static int TARGET_CONTAINER_TCP_PORT = ConfigurationTestUtils.CONTAINER_TCP_PORT_0 + 1;

    private final static String TARGET_PASS_PHRASE = "pass-phrase";

    @Rule
    public final PetalsJmxApiJunitRule jmxApiOnTargetContainer = new PetalsJmxApiJunitRule(TARGET_JMX_HOST,
            TARGET_JMX_PORT, null, null);

    @Rule
    public final LocalServerPropertiesBuilder localServerPropertiesBuilder = new LocalServerPropertiesBuilder();

    @Rule
    public final LocalServerPropertiesBuilder targetServerPropertiesBuilder = new LocalServerPropertiesBuilder();

    /**
     * <p>
     * Check {@link TopologyServiceImpl#attachContainerTo(ContainerConfiguration, String)} against:
     * <ul>
     * <li>no container configuration is provided</li>
     * </ul>
     * </p>
     * <p>
     * Expected results: An error occurs about the missing container configuration
     * </p>
     */
    @Test(expected = NoContainerConfigurationProvidedException.class)
    public void testNoContainerConfiguration() throws PetalsException, InvocationTargetException, IOException {

        // Create a basic sample topology
        final Topology basicCurrentTopology = ConfigurationTestUtils.createBasicSampleTopology();

        // Create and initialize the configuration service
        final Properties currentServerProps = this.localServerPropertiesBuilder
                .createMinimalServerProperties(ConfigurationTestUtils.CONTAINER_NAME_0);
        final ConfigurationService cfgSvc = new ConfigurationServiceImpl();
        ConfigurationTestUtils.startConfigurationService(cfgSvc);
        try {
            cfgSvc.loadConfiguration(currentServerProps, basicCurrentTopology);

            final TopologyService topoSvc = new TopologyServiceImpl();
            ReflectionHelper.setPrivateField(topoSvc, "configurationService", cfgSvc);
            this.startTopologyService(topoSvc);

            try {
                topoSvc.attachContainerTo(null, null);
            } finally {
                this.stopTopologyService(topoSvc);
            }
        } finally {
            ConfigurationTestUtils.stopConfigurationService(cfgSvc);
        }
    }

    /**
     * <p>
     * Check {@link TopologyServiceImpl#attachContainerTo(ContainerConfiguration, String)} against:
     * <ul>
     * <li>no container name is provided in the given container configuration</li>
     * </ul>
     * </p>
     * <p>
     * Expected results: An error occurs about the missing container name
     * </p>
     */
    @Test(expected = NoContainerNameProvidedException.class)
    public void testNoContainerName() throws PetalsException, InvocationTargetException, IOException {

        // Create a basic sample topology
        final Topology basicCurrentTopology = ConfigurationTestUtils.createBasicSampleTopology();

        // Create and initialize the configuration service
        final Properties currentServerProps = this.localServerPropertiesBuilder
                .createMinimalServerProperties(ConfigurationTestUtils.CONTAINER_NAME_0);
        final ConfigurationService cfgSvc = new ConfigurationServiceImpl();
        ConfigurationTestUtils.startConfigurationService(cfgSvc);
        try {
            cfgSvc.loadConfiguration(currentServerProps, basicCurrentTopology);

            final TopologyService topoSvc = new TopologyServiceImpl();
            ReflectionHelper.setPrivateField(topoSvc, "configurationService", cfgSvc);
            this.startTopologyService(topoSvc);

            try {
                final ContainerConfiguration targetContainerConfiguration = new ContainerConfiguration();

                topoSvc.attachContainerTo(targetContainerConfiguration, TARGET_PASS_PHRASE);
            } finally {
                this.stopTopologyService(topoSvc);
            }
        } finally {
            ConfigurationTestUtils.stopConfigurationService(cfgSvc);
        }
    }

    /**
     * <p>
     * Check {@link TopologyServiceImpl#attachContainerTo(ContainerConfiguration, String)} against:
     * <ul>
     * <li>no security pass-phrase is provided</li>
     * </ul>
     * </p>
     * <p>
     * Expected results: An error occurs about the missing security pass-phrase
     * </p>
     */
    @Test(expected = NoSecurityPassPhraseProvidedException.class)
    public void testNoSecurityPassPhrase() throws PetalsException, InvocationTargetException, IOException {

        // Create a basic sample topology
        final Topology basicCurrentTopology = ConfigurationTestUtils.createBasicSampleTopology();

        // Create and initialize the configuration service
        final Properties currentServerProps = this.localServerPropertiesBuilder
                .createMinimalServerProperties(ConfigurationTestUtils.CONTAINER_NAME_0);
        final ConfigurationService cfgSvc = new ConfigurationServiceImpl();
        ConfigurationTestUtils.startConfigurationService(cfgSvc);
        try {
            cfgSvc.loadConfiguration(currentServerProps, basicCurrentTopology);

            final TopologyService topoSvc = new TopologyServiceImpl();
            ReflectionHelper.setPrivateField(topoSvc, "configurationService", cfgSvc);
            this.startTopologyService(topoSvc);

            try {
                final ContainerConfiguration targetContainerConfiguration = new ContainerConfiguration();
                targetContainerConfiguration.setName(TARGET_CONTAINER_NAME);

                topoSvc.attachContainerTo(targetContainerConfiguration, null);
            } finally {
                this.stopTopologyService(topoSvc);
            }
        } finally {
            ConfigurationTestUtils.stopConfigurationService(cfgSvc);
        }
    }

    /**
     * <p>
     * Check {@link TopologyServiceImpl#attachContainerTo(ContainerConfiguration, String)} against:
     * <ul>
     * <li>an empty security pass-phrase is provided</li>
     * </ul>
     * </p>
     * <p>
     * Expected results: An error occurs about the missing security pass-phrase
     * </p>
     */
    @Test(expected = NoSecurityPassPhraseProvidedException.class)
    public void testEmptySecurityPassPhrase() throws PetalsException, InvocationTargetException, IOException {

        // Create a basic sample topology
        final Topology basicCurrentTopology = ConfigurationTestUtils.createBasicSampleTopology();

        // Create and initialize the configuration service
        final Properties currentServerProps = this.localServerPropertiesBuilder
                .createMinimalServerProperties(ConfigurationTestUtils.CONTAINER_NAME_0);
        final ConfigurationService cfgSvc = new ConfigurationServiceImpl();
        ConfigurationTestUtils.startConfigurationService(cfgSvc);
        try {
            cfgSvc.loadConfiguration(currentServerProps, basicCurrentTopology);

            final TopologyService topoSvc = new TopologyServiceImpl();
            ReflectionHelper.setPrivateField(topoSvc, "configurationService", cfgSvc);
            this.startTopologyService(topoSvc);

            try {
                final ContainerConfiguration targetContainerConfiguration = new ContainerConfiguration();
                targetContainerConfiguration.setName(TARGET_CONTAINER_NAME);

                topoSvc.attachContainerTo(targetContainerConfiguration, "");
            } finally {
                this.stopTopologyService(topoSvc);
            }
        } finally {
            ConfigurationTestUtils.stopConfigurationService(cfgSvc);
        }
    }

    /**
     * <p>
     * Check {@link TopologyServiceImpl#attachContainerTo(ContainerConfiguration, String)} against:
     * <ul>
     * <li>the sub-domain of the given container configuration is not set</li>
     * </ul>
     * </p>
     * <p>
     * Expected results: An error occurs about the missing sub-domain information of the given container
     * </p>
     */
    @Test(expected = NoSubdomainConfigurationProvidedException.class)
    public void testNoSubdomain() throws PetalsException, InvocationTargetException, IOException {

        // Create a basic sample topology
        final Topology basicCurrentTopology = ConfigurationTestUtils.createBasicSampleTopology();

        // Create and initialize the configuration service
        final Properties currentServerProps = this.localServerPropertiesBuilder
                .createMinimalServerProperties(ConfigurationTestUtils.CONTAINER_NAME_0);
        final ConfigurationService cfgSvc = new ConfigurationServiceImpl();
        ConfigurationTestUtils.startConfigurationService(cfgSvc);
        try {
            cfgSvc.loadConfiguration(currentServerProps, basicCurrentTopology);

            final TopologyService topoSvc = new TopologyServiceImpl();
            ReflectionHelper.setPrivateField(topoSvc, "configurationService", cfgSvc);
            this.startTopologyService(topoSvc);

            try {
                final ContainerConfiguration targetContainerConfiguration = new ContainerConfiguration();
                targetContainerConfiguration.setName(TARGET_CONTAINER_NAME);

                topoSvc.attachContainerTo(targetContainerConfiguration, TARGET_PASS_PHRASE);
            } finally {
                this.stopTopologyService(topoSvc);
            }
        } finally {
            ConfigurationTestUtils.stopConfigurationService(cfgSvc);
        }
    }

    /**
     * <p>
     * Check {@link TopologyServiceImpl#attachContainerTo(ContainerConfiguration, String)} against:
     * <ul>
     * <li>the sub-domain of the current container is static</li>
     * </ul>
     * </p>
     * <p>
     * Expected results: An error occurs because the sub-domain mode is not <b>dynamic</b>
     * </p>
     */
    @Test(expected = SubdomainIsNotDynamicException.class)
    public void testCurrentSubdomainNotDynamic() throws PetalsException, InvocationTargetException, IOException {

        // Create a basic sample topology
        final Topology basicCurrentTopology = ConfigurationTestUtils.createBasicSampleTopology();
        basicCurrentTopology.getDomain().getSubDomain().get(0).setMode(SubdomainMode.STATIC);

        // Create and initialize the configuration service
        final Properties currentServerProps = this.localServerPropertiesBuilder
                .createMinimalServerProperties(ConfigurationTestUtils.CONTAINER_NAME_0);
        final ConfigurationService cfgSvc = new ConfigurationServiceImpl();
        ConfigurationTestUtils.startConfigurationService(cfgSvc);
        try {
            cfgSvc.loadConfiguration(currentServerProps, basicCurrentTopology);

            final TopologyService topoSvc = new TopologyServiceImpl();
            ReflectionHelper.setPrivateField(topoSvc, "configurationService", cfgSvc);
            this.startTopologyService(topoSvc);

            try {
                final ContainerConfiguration targetContainerConfiguration = new ContainerConfiguration();
                targetContainerConfiguration.setName(TARGET_CONTAINER_NAME);
                targetContainerConfiguration.setSubdomainName(TARGET_SUBDOMAIN_NAME);
                targetContainerConfiguration.setHost(TARGET_JMX_HOST);
                targetContainerConfiguration.setJmxRMIConnectorPort(TARGET_JMX_PORT);

                topoSvc.attachContainerTo(targetContainerConfiguration, TARGET_PASS_PHRASE);
            } finally {
                this.stopTopologyService(topoSvc);
            }
        } finally {
            ConfigurationTestUtils.stopConfigurationService(cfgSvc);
        }
    }

    /**
     * <p>
     * Check {@link TopologyServiceImpl#attachContainerTo(ContainerConfiguration, String)} against:
     * <ul>
     * <li>the target container is not running</li>
     * </ul>
     * </p>
     * <p>
     * Expected results: An error occurs because the sub-domain mode is not <b>dynamic</b>
     * </p>
     */
    @Test(expected = ConnectionErrorException.class)
    public void testTargetContainerNotRunning() throws Throwable {

        // Create a basic sample topology
        final Topology basicCurrentTopology = ConfigurationTestUtils.createBasicSampleTopology();
        basicCurrentTopology.getDomain().getSubDomain().get(0).setMode(SubdomainMode.STATIC);

        // Create and initialize the configuration service
        final Properties currentServerProps = this.localServerPropertiesBuilder
                .createMinimalServerProperties(ConfigurationTestUtils.CONTAINER_NAME_0);
        final ConfigurationService cfgSvc = new ConfigurationServiceImpl();
        ConfigurationTestUtils.startConfigurationService(cfgSvc);
        try {
            cfgSvc.loadConfiguration(currentServerProps, basicCurrentTopology);

            final TopologyService topoSvc = new TopologyServiceImpl();
            ReflectionHelper.setPrivateField(topoSvc, "configurationService", cfgSvc);
            this.startTopologyService(topoSvc);

            try {
                final ContainerConfiguration targetContainerConfiguration = new ContainerConfiguration();
                targetContainerConfiguration.setName(TARGET_CONTAINER_NAME);
                targetContainerConfiguration.setSubdomainName(TARGET_SUBDOMAIN_NAME);
                targetContainerConfiguration.setHost("host-not-running");
                targetContainerConfiguration.setJmxRMIConnectorPort(TARGET_JMX_PORT);

                topoSvc.attachContainerTo(targetContainerConfiguration, TARGET_PASS_PHRASE);
            } catch (final AttachContainerException e) {
                // It's the expected direct exception. The error connection exception is the cause of this exception
                throw e.getCause();
            } finally {
                this.stopTopologyService(topoSvc);
            }
        } finally {
            ConfigurationTestUtils.stopConfigurationService(cfgSvc);
        }
    }

    /**
     * <p>
     * Check {@link TopologyServiceImpl#attachContainerTo(ContainerConfiguration, String)} against:
     * <ul>
     * <li>the sub-domain in which the container must be attached is not the expected one</li>
     * </ul>
     * </p>
     * <p>
     * Expected results: An error occurs about the inconsistency about sub-domain name
     * </p>
     */
    @Test(expected = InconsistentSubdomainNameException.class)
    public void testTargetSubdomainNotFound() throws InvocationTargetException, PetalsException, IOException {
        
        // Create a basic target topology for the remote container
        final Topology basicTargetTopology = TopologyServiceImpl_AttachContainerToTest.createBasicTargetTopology();
        final Properties targetServerProps = this.targetServerPropertiesBuilder
                .createMinimalServerProperties(TARGET_CONTAINER_NAME);
        final ConfigurationService targetCfgSvc = new ConfigurationServiceImpl();
        ConfigurationTestUtils.startConfigurationService(targetCfgSvc);
        try {
            targetCfgSvc.loadConfiguration(targetServerProps, basicTargetTopology);
            final PetalsCompositeController targetPetalsCompositeCtrl = EasyMock
                    .createMock(PetalsCompositeController.class);
            EasyMock.expect(targetPetalsCompositeCtrl.getRegistryConfiguration(TARGET_PASS_PHRASE))
                    .andReturn(new Properties()).anyTimes();
            EasyMock.replay(targetPetalsCompositeCtrl);
            final TopologyService targetTopoSvc = new TopologyServiceImpl();
            ReflectionHelper.setPrivateField(targetTopoSvc, "configurationService", targetCfgSvc);
            ReflectionHelper.setPrivateField(targetTopoSvc, "petalsCompositeController", targetPetalsCompositeCtrl);
            this.startTopologyService(targetTopoSvc);
            try {
                // Create a basic sample topology for the current container
                final Topology basicCurrentTopology = ConfigurationTestUtils.createBasicSampleTopology();
                basicCurrentTopology.getDomain().getSubDomain().get(0).setMode(SubdomainMode.DYNAMIC);
        
                // Create and initialize the configuration service
                final Properties currentServerProps = this.localServerPropertiesBuilder
                        .createMinimalServerProperties(ConfigurationTestUtils.CONTAINER_NAME_0);
                final ConfigurationService currentCfgSvc = new ConfigurationServiceImpl();
                ConfigurationTestUtils.startConfigurationService(currentCfgSvc);
                try {
                    currentCfgSvc.loadConfiguration(currentServerProps, basicCurrentTopology);
        
                    final TopologyService currentTopoSvc = new TopologyServiceImpl();
                    ReflectionHelper.setPrivateField(currentTopoSvc, "configurationService", currentCfgSvc);
                    this.startTopologyService(currentTopoSvc);
        
                    try {
                        final ContainerConfiguration targetContainerConfiguration = new ContainerConfiguration();
                        targetContainerConfiguration.setName(TARGET_CONTAINER_NAME);
                        targetContainerConfiguration.setSubdomainName("inconsistent-subdomain-name");
                        targetContainerConfiguration.setHost(TARGET_JMX_HOST);
                        targetContainerConfiguration.setJmxRMIConnectorPort(TARGET_JMX_PORT);
                        
                        final TopologyServiceClient jmxTopologyServiceClientMock = EasyMock
                                .createMock(TopologyServiceClient.class);
                        EasyMock.expect(jmxTopologyServiceClientMock.retrieveTopology(TARGET_PASS_PHRASE))
                                .andReturn(targetTopoSvc.retrieveTopology(TARGET_PASS_PHRASE)).anyTimes();
                        EasyMock.replay(jmxTopologyServiceClientMock);
                        this.jmxApiOnTargetContainer.setTopologyServiceClient(jmxTopologyServiceClientMock);
        
                        currentTopoSvc.attachContainerTo(targetContainerConfiguration, TARGET_PASS_PHRASE);
                    } finally {
                        this.stopTopologyService(currentTopoSvc);
                    }
                } finally {
                    ConfigurationTestUtils.stopConfigurationService(currentCfgSvc);
                }
            } finally {
                this.stopTopologyService(targetTopoSvc);
            }
        } finally {
            ConfigurationTestUtils.stopConfigurationService(targetCfgSvc);
        }
    }

    /**
     * <p>
     * Check {@link TopologyServiceImpl#attachContainerTo(ContainerConfiguration, String)} against:
     * <ul>
     * <li>the given container is not found in the target topology</li>
     * </ul>
     * </p>
     * <p>
     * Expected results: An error occurs about the inconsistency about the container name
     * </p>
     */
    @Test(expected = InconsistentContainerNameException.class)
    public void testTargetContainerNotFound() throws InvocationTargetException, PetalsException, IOException {

        // Create a basic target topology for the remote container
        final Topology basicTargetTopology = TopologyServiceImpl_AttachContainerToTest.createBasicTargetTopology();
        final Properties targetServerProps = this.targetServerPropertiesBuilder
                .createMinimalServerProperties(TARGET_CONTAINER_NAME);
        final ConfigurationService targetCfgSvc = new ConfigurationServiceImpl();
        ConfigurationTestUtils.startConfigurationService(targetCfgSvc);
        try {
            targetCfgSvc.loadConfiguration(targetServerProps, basicTargetTopology);
            final PetalsCompositeController targetPetalsCompositeCtrl = EasyMock
                    .createMock(PetalsCompositeController.class);
            EasyMock.expect(targetPetalsCompositeCtrl.getRegistryConfiguration(TARGET_PASS_PHRASE))
                    .andReturn(new Properties()).anyTimes();
            EasyMock.replay(targetPetalsCompositeCtrl);
            final TopologyService targetTopoSvc = new TopologyServiceImpl();
            ReflectionHelper.setPrivateField(targetTopoSvc, "configurationService", targetCfgSvc);
            ReflectionHelper.setPrivateField(targetTopoSvc, "petalsCompositeController", targetPetalsCompositeCtrl);
            this.startTopologyService(targetTopoSvc);
            try {
                // Create a basic sample topology for the current container
                final Topology basicCurrentTopology = ConfigurationTestUtils.createBasicSampleTopology();
                basicCurrentTopology.getDomain().getSubDomain().get(0).setMode(SubdomainMode.DYNAMIC);

                // Create and initialize the configuration service
                final Properties currentServerProps = this.localServerPropertiesBuilder
                        .createMinimalServerProperties(ConfigurationTestUtils.CONTAINER_NAME_0);
                final ConfigurationService currentCfgSvc = new ConfigurationServiceImpl();
                ConfigurationTestUtils.startConfigurationService(currentCfgSvc);
                try {
                    currentCfgSvc.loadConfiguration(currentServerProps, basicCurrentTopology);

                    final TopologyService currentTopoSvc = new TopologyServiceImpl();
                    ReflectionHelper.setPrivateField(currentTopoSvc, "configurationService", currentCfgSvc);
                    this.startTopologyService(currentTopoSvc);

                    try {
                        final ContainerConfiguration targetContainerConfiguration = new ContainerConfiguration();
                        targetContainerConfiguration.setName("target-container-name-not-exist");
                        targetContainerConfiguration.setSubdomainName(TARGET_SUBDOMAIN_NAME);
                        targetContainerConfiguration.setHost(TARGET_JMX_HOST);
                        targetContainerConfiguration.setJmxRMIConnectorPort(TARGET_JMX_PORT);

                        final TopologyServiceClient jmxTopologyServiceClientMock = EasyMock
                                .createMock(TopologyServiceClient.class);
                        EasyMock.expect(jmxTopologyServiceClientMock.retrieveTopology(TARGET_PASS_PHRASE))
                                .andReturn(targetTopoSvc.retrieveTopology(TARGET_PASS_PHRASE)).anyTimes();
                        EasyMock.replay(jmxTopologyServiceClientMock);
                        this.jmxApiOnTargetContainer.setTopologyServiceClient(jmxTopologyServiceClientMock);

                        currentTopoSvc.attachContainerTo(targetContainerConfiguration, TARGET_PASS_PHRASE);
                    } finally {
                        this.stopTopologyService(currentTopoSvc);
                    }
                } finally {
                    ConfigurationTestUtils.stopConfigurationService(currentCfgSvc);
                }
            } finally {
                this.stopTopologyService(targetTopoSvc);
            }
        } finally {
            ConfigurationTestUtils.stopConfigurationService(targetCfgSvc);
        }
    }

    /**
     * <p>
     * Check {@link TopologyServiceImpl#attachContainerTo(ContainerConfiguration, String)} against:
     * <ul>
     * <li>the sub-domain of the remote container is static</li>
     * </ul>
     * </p>
     * <p>
     * Expected results: An error occurs because the remote sub-domain mode is not <b>dynamic</b>
     * </p>
     */
    @Test(expected = SubdomainIsNotDynamicException.class)
    public void testTargetSubdomainNotDynamic() throws PetalsException, InvocationTargetException, IOException {

        // Create a basic target topology for the remote container
        final Topology basicTargetTopology = TopologyServiceImpl_AttachContainerToTest.createBasicTargetTopology();
        basicTargetTopology.getDomain().getSubDomain().get(0).setMode(SubdomainMode.STATIC);
        final Properties targetServerProps = this.targetServerPropertiesBuilder
                .createMinimalServerProperties(TARGET_CONTAINER_NAME);
        final ConfigurationService targetCfgSvc = new ConfigurationServiceImpl();
        ConfigurationTestUtils.startConfigurationService(targetCfgSvc);
        try {
            targetCfgSvc.loadConfiguration(targetServerProps, basicTargetTopology);
            final PetalsCompositeController targetPetalsCompositeCtrl = EasyMock
                    .createMock(PetalsCompositeController.class);
            EasyMock.expect(targetPetalsCompositeCtrl.getRegistryConfiguration(TARGET_PASS_PHRASE))
                    .andReturn(new Properties()).anyTimes();
            EasyMock.replay(targetPetalsCompositeCtrl);
            final TopologyService targetTopoSvc = new TopologyServiceImpl();
            ReflectionHelper.setPrivateField(targetTopoSvc, "configurationService", targetCfgSvc);
            ReflectionHelper.setPrivateField(targetTopoSvc, "petalsCompositeController", targetPetalsCompositeCtrl);
            this.startTopologyService(targetTopoSvc);
            try {
                // Create a basic sample topology for the current container
                final Topology basicCurrentTopology = ConfigurationTestUtils.createBasicSampleTopology();
                basicCurrentTopology.getDomain().getSubDomain().get(0).setMode(SubdomainMode.DYNAMIC);

                // Create and initialize the configuration service
                final Properties currentServerProps = this.localServerPropertiesBuilder
                        .createMinimalServerProperties(ConfigurationTestUtils.CONTAINER_NAME_0);
                final ConfigurationService currentCfgSvc = new ConfigurationServiceImpl();
                ConfigurationTestUtils.startConfigurationService(currentCfgSvc);
                try {
                    currentCfgSvc.loadConfiguration(currentServerProps, basicCurrentTopology);

                    final TopologyService currentTopoSvc = new TopologyServiceImpl();
                    ReflectionHelper.setPrivateField(currentTopoSvc, "configurationService", currentCfgSvc);
                    this.startTopologyService(currentTopoSvc);

                    try {
                        final ContainerConfiguration targetContainerConfiguration = new ContainerConfiguration();
                        targetContainerConfiguration.setName(TARGET_CONTAINER_NAME);
                        targetContainerConfiguration.setSubdomainName(TARGET_SUBDOMAIN_NAME);
                        targetContainerConfiguration.setHost(TARGET_JMX_HOST);
                        targetContainerConfiguration.setJmxRMIConnectorPort(TARGET_JMX_PORT);

                        final TopologyServiceClient jmxTopologyServiceClientMock = EasyMock
                                .createMock(TopologyServiceClient.class);
                        EasyMock.expect(jmxTopologyServiceClientMock.retrieveTopology(TARGET_PASS_PHRASE))
                                .andReturn(targetTopoSvc.retrieveTopology(TARGET_PASS_PHRASE)).anyTimes();
                        EasyMock.replay(jmxTopologyServiceClientMock);
                        this.jmxApiOnTargetContainer.setTopologyServiceClient(jmxTopologyServiceClientMock);

                        currentTopoSvc.attachContainerTo(targetContainerConfiguration, TARGET_PASS_PHRASE);
                    } finally {
                        this.stopTopologyService(currentTopoSvc);
                    }
                } finally {
                    ConfigurationTestUtils.stopConfigurationService(currentCfgSvc);
                }
            } finally {
                this.stopTopologyService(targetTopoSvc);
            }
        } finally {
            ConfigurationTestUtils.stopConfigurationService(targetCfgSvc);
        }
    }

    /**
     * <p>
     * Check {@link TopologyServiceImpl#attachContainerTo(ContainerConfiguration, String)} against:
     * <ul>
     * <li>the registry implementation of the current sub-domain is different from the one of the target sub-domain: no
     * registry implementation defined for the current sub-domain, an explicit registry implementation is set to the
     * target sub-domain</li>
     * </ul>
     * </p>
     * <p>
     * Expected results: An error occurs because the registry implementations are inconsistent
     * </p>
     */
    @Test(expected = InconsistentSubdomainRegistryImplException.class)
    public void testRegistryImplInconsistent_NoImplOnCur_AnImpOnTarget() throws PetalsException,
            InvocationTargetException, IOException {

        // Create a basic target topology for the remote container
        final Topology basicTargetTopology = TopologyServiceImpl_AttachContainerToTest.createBasicTargetTopology();
        final Subdomain targetSubdomain = basicTargetTopology.getDomain().getSubDomain().get(0);
        targetSubdomain.setMode(SubdomainMode.DYNAMIC);
        targetSubdomain.setRegistryImplementation("a.registry.implementation");
        final Properties targetServerProps = this.targetServerPropertiesBuilder
                .createMinimalServerProperties(TARGET_CONTAINER_NAME);
        final ConfigurationService targetCfgSvc = new ConfigurationServiceImpl();
        ConfigurationTestUtils.startConfigurationService(targetCfgSvc);
        try {
            targetCfgSvc.loadConfiguration(targetServerProps, basicTargetTopology);
            final PetalsCompositeController targetPetalsCompositeCtrl = EasyMock
                    .createMock(PetalsCompositeController.class);
            EasyMock.expect(targetPetalsCompositeCtrl.getRegistryConfiguration(TARGET_PASS_PHRASE))
                    .andReturn(new Properties()).anyTimes();
            EasyMock.replay(targetPetalsCompositeCtrl);
            final TopologyService targetTopoSvc = new TopologyServiceImpl();
            ReflectionHelper.setPrivateField(targetTopoSvc, "configurationService", targetCfgSvc);
            ReflectionHelper.setPrivateField(targetTopoSvc, "petalsCompositeController", targetPetalsCompositeCtrl);
            this.startTopologyService(targetTopoSvc);
            try {
                // Create a basic sample topology for the current container
                final Topology basicCurrentTopology = ConfigurationTestUtils.createBasicSampleTopology();
                basicCurrentTopology.getDomain().getSubDomain().get(0).setMode(SubdomainMode.DYNAMIC);

                // Create and initialize the configuration service
                final Properties currentServerProps = this.localServerPropertiesBuilder
                        .createMinimalServerProperties(ConfigurationTestUtils.CONTAINER_NAME_0);
                final ConfigurationService currentCfgSvc = new ConfigurationServiceImpl();
                ConfigurationTestUtils.startConfigurationService(currentCfgSvc);
                try {
                    currentCfgSvc.loadConfiguration(currentServerProps, basicCurrentTopology);

                    final TopologyService currentTopoSvc = new TopologyServiceImpl();
                    ReflectionHelper.setPrivateField(currentTopoSvc, "configurationService", currentCfgSvc);
                    this.startTopologyService(currentTopoSvc);

                    try {
                        final ContainerConfiguration targetContainerConfiguration = new ContainerConfiguration();
                        targetContainerConfiguration.setName(TARGET_CONTAINER_NAME);
                        targetContainerConfiguration.setSubdomainName(TARGET_SUBDOMAIN_NAME);
                        targetContainerConfiguration.setHost(TARGET_JMX_HOST);
                        targetContainerConfiguration.setJmxRMIConnectorPort(TARGET_JMX_PORT);

                        final TopologyServiceClient jmxTopologyServiceClientMock = EasyMock
                                .createMock(TopologyServiceClient.class);
                        EasyMock.expect(jmxTopologyServiceClientMock.retrieveTopology(TARGET_PASS_PHRASE))
                                .andReturn(targetTopoSvc.retrieveTopology(TARGET_PASS_PHRASE)).anyTimes();
                        EasyMock.replay(jmxTopologyServiceClientMock);
                        this.jmxApiOnTargetContainer.setTopologyServiceClient(jmxTopologyServiceClientMock);

                        currentTopoSvc.attachContainerTo(targetContainerConfiguration, TARGET_PASS_PHRASE);
                    } finally {
                        this.stopTopologyService(currentTopoSvc);
                    }
                } finally {
                    ConfigurationTestUtils.stopConfigurationService(currentCfgSvc);
                }
            } finally {
                this.stopTopologyService(targetTopoSvc);
            }
        } finally {
            ConfigurationTestUtils.stopConfigurationService(targetCfgSvc);
        }
    }

    /**
     * <p>
     * Check {@link TopologyServiceImpl#attachContainerTo(ContainerConfiguration, String)} against:
     * <ul>
     * <li>the registry implementation of the current sub-domain is different from the one of the target sub-domain: an
     * explicit registry implementation is set to the current sub-domain, no registry implementation defined for the
     * target sub-domain</li>
     * </ul>
     * </p>
     * <p>
     * Expected results: An error occurs because the registry implementations are inconsistent
     * </p>
     */
    @Test(expected = InconsistentSubdomainRegistryImplException.class)
    public void testRegistryImplInconsistent_AnImplOnCur_NoImpOnTarget() throws PetalsException,
            InvocationTargetException, IOException {

        // Create a basic target topology for the remote container
        final Topology basicTargetTopology = TopologyServiceImpl_AttachContainerToTest.createBasicTargetTopology();
        basicTargetTopology.getDomain().getSubDomain().get(0).setMode(SubdomainMode.DYNAMIC);
        final Properties targetServerProps = this.targetServerPropertiesBuilder
                .createMinimalServerProperties(TARGET_CONTAINER_NAME);
        final ConfigurationService targetCfgSvc = new ConfigurationServiceImpl();
        ConfigurationTestUtils.startConfigurationService(targetCfgSvc);
        try {
            targetCfgSvc.loadConfiguration(targetServerProps, basicTargetTopology);
            final PetalsCompositeController targetPetalsCompositeCtrl = EasyMock
                    .createMock(PetalsCompositeController.class);
            EasyMock.expect(targetPetalsCompositeCtrl.getRegistryConfiguration(TARGET_PASS_PHRASE))
                    .andReturn(new Properties()).anyTimes();
            EasyMock.replay(targetPetalsCompositeCtrl);
            final TopologyService targetTopoSvc = new TopologyServiceImpl();
            ReflectionHelper.setPrivateField(targetTopoSvc, "configurationService", targetCfgSvc);
            ReflectionHelper.setPrivateField(targetTopoSvc, "petalsCompositeController", targetPetalsCompositeCtrl);
            this.startTopologyService(targetTopoSvc);
            try {
                // Create a basic sample topology for the current container
                final Topology basicCurrentTopology = ConfigurationTestUtils.createBasicSampleTopology();
                final Subdomain currentSubdomain = basicCurrentTopology.getDomain().getSubDomain().get(0);
                currentSubdomain.setMode(SubdomainMode.DYNAMIC);
                currentSubdomain.setRegistryImplementation("a.registry.implementation");

                // Create and initialize the configuration service
                final Properties currentServerProps = this.localServerPropertiesBuilder
                        .createMinimalServerProperties(ConfigurationTestUtils.CONTAINER_NAME_0);
                final ConfigurationService currentCfgSvc = new ConfigurationServiceImpl();
                ConfigurationTestUtils.startConfigurationService(currentCfgSvc);
                try {
                    currentCfgSvc.loadConfiguration(currentServerProps, basicCurrentTopology);

                    final TopologyService currentTopoSvc = new TopologyServiceImpl();
                    ReflectionHelper.setPrivateField(currentTopoSvc, "configurationService", currentCfgSvc);
                    this.startTopologyService(currentTopoSvc);

                    try {
                        final ContainerConfiguration targetContainerConfiguration = new ContainerConfiguration();
                        targetContainerConfiguration.setName(TARGET_CONTAINER_NAME);
                        targetContainerConfiguration.setSubdomainName(TARGET_SUBDOMAIN_NAME);
                        targetContainerConfiguration.setHost(TARGET_JMX_HOST);
                        targetContainerConfiguration.setJmxRMIConnectorPort(TARGET_JMX_PORT);

                        final TopologyServiceClient jmxTopologyServiceClientMock = EasyMock
                                .createMock(TopologyServiceClient.class);
                        EasyMock.expect(jmxTopologyServiceClientMock.retrieveTopology(TARGET_PASS_PHRASE))
                                .andReturn(targetTopoSvc.retrieveTopology(TARGET_PASS_PHRASE)).anyTimes();

                        EasyMock.replay(jmxTopologyServiceClientMock);
                        this.jmxApiOnTargetContainer.setTopologyServiceClient(jmxTopologyServiceClientMock);

                        currentTopoSvc.attachContainerTo(targetContainerConfiguration, TARGET_PASS_PHRASE);
                    } finally {
                        this.stopTopologyService(currentTopoSvc);
                    }
                } finally {
                    ConfigurationTestUtils.stopConfigurationService(currentCfgSvc);
                }
            } finally {
                this.stopTopologyService(targetTopoSvc);
            }
        } finally {
            ConfigurationTestUtils.stopConfigurationService(targetCfgSvc);
        }
    }

    /**
     * <p>
     * Check {@link TopologyServiceImpl#attachContainerTo(ContainerConfiguration, String)} against:
     * <ul>
     * <li>the registry implementation of the current sub-domain is different from the one of the target sub-domain: an
     * explicit registry implementation is set to the current sub-domain, another explicit registry implementation is
     * set to the target sub-domain</li>
     * </ul>
     * </p>
     * <p>
     * Expected results: An error occurs because the registry implementations are inconsistent
     * </p>
     */
    @Test(expected = InconsistentSubdomainRegistryImplException.class)
    public void testRegistryImplInconsistent_DifferentImplSet() throws PetalsException, InvocationTargetException,
            IOException {

        // Create a basic target topology for the remote container
        final Topology basicTargetTopology = TopologyServiceImpl_AttachContainerToTest.createBasicTargetTopology();
        final Subdomain targetSubdomain = basicTargetTopology.getDomain().getSubDomain().get(0);
        targetSubdomain.setMode(SubdomainMode.DYNAMIC);
        targetSubdomain.setRegistryImplementation("another.registry.implementation");
        final Properties targetServerProps = this.targetServerPropertiesBuilder
                .createMinimalServerProperties(TARGET_CONTAINER_NAME);
        final ConfigurationService targetCfgSvc = new ConfigurationServiceImpl();
        ConfigurationTestUtils.startConfigurationService(targetCfgSvc);
        try {
            targetCfgSvc.loadConfiguration(targetServerProps, basicTargetTopology);
            final PetalsCompositeController targetPetalsCompositeCtrl = EasyMock
                    .createMock(PetalsCompositeController.class);
            EasyMock.expect(targetPetalsCompositeCtrl.getRegistryConfiguration(TARGET_PASS_PHRASE))
                    .andReturn(new Properties()).anyTimes();
            EasyMock.replay(targetPetalsCompositeCtrl);
            final TopologyService targetTopoSvc = new TopologyServiceImpl();
            ReflectionHelper.setPrivateField(targetTopoSvc, "configurationService", targetCfgSvc);
            ReflectionHelper.setPrivateField(targetTopoSvc, "petalsCompositeController", targetPetalsCompositeCtrl);
            this.startTopologyService(targetTopoSvc);
            try {
                // Create a basic sample topology for the current container
                final Topology basicCurrentTopology = ConfigurationTestUtils.createBasicSampleTopology();
                final Subdomain currentSubdomain = basicCurrentTopology.getDomain().getSubDomain().get(0);
                currentSubdomain.setMode(SubdomainMode.DYNAMIC);
                currentSubdomain.setRegistryImplementation("a.registry.implementation");

                // Create and initialize the configuration service
                final Properties currentServerProps = this.localServerPropertiesBuilder
                        .createMinimalServerProperties(ConfigurationTestUtils.CONTAINER_NAME_0);
                final ConfigurationService currentCfgSvc = new ConfigurationServiceImpl();
                ConfigurationTestUtils.startConfigurationService(currentCfgSvc);
                try {
                    currentCfgSvc.loadConfiguration(currentServerProps, basicCurrentTopology);

                    final TopologyService currentTopoSvc = new TopologyServiceImpl();
                    ReflectionHelper.setPrivateField(currentTopoSvc, "configurationService", currentCfgSvc);
                    this.startTopologyService(currentTopoSvc);

                    try {
                        final ContainerConfiguration targetContainerConfiguration = new ContainerConfiguration();
                        targetContainerConfiguration.setName(TARGET_CONTAINER_NAME);
                        targetContainerConfiguration.setSubdomainName(TARGET_SUBDOMAIN_NAME);
                        targetContainerConfiguration.setHost(TARGET_JMX_HOST);
                        targetContainerConfiguration.setJmxRMIConnectorPort(TARGET_JMX_PORT);

                        final TopologyServiceClient jmxTopologyServiceClientMock = EasyMock
                                .createMock(TopologyServiceClient.class);
                        EasyMock.expect(jmxTopologyServiceClientMock.retrieveTopology(TARGET_PASS_PHRASE))
                                .andReturn(targetTopoSvc.retrieveTopology(TARGET_PASS_PHRASE)).anyTimes();
                        EasyMock.replay(jmxTopologyServiceClientMock);
                        this.jmxApiOnTargetContainer.setTopologyServiceClient(jmxTopologyServiceClientMock);

                        currentTopoSvc.attachContainerTo(targetContainerConfiguration, TARGET_PASS_PHRASE);
                    } finally {
                        this.stopTopologyService(currentTopoSvc);
                    }
                } finally {
                    ConfigurationTestUtils.stopConfigurationService(currentCfgSvc);
                }
            } finally {
                this.stopTopologyService(targetTopoSvc);
            }
        } finally {
            ConfigurationTestUtils.stopConfigurationService(targetCfgSvc);
        }
    }

    /**
     * <p>
     * Check {@link TopologyServiceImpl#attachContainerTo(ContainerConfiguration, String)} against:
     * <ul>
     * <li>all is ok</li>
     * </ul>
     * </p>
     * <p>
     * Expected results:
     * <ul>
     * <li>No error occurs</li>
     * <li>...
     * <li>
     * </ul>
     * </p>
     */
    @Test
    public void testAllIsOk() throws PetalsException, InvocationTargetException, RegistryException,
            InterruptedException, IOException {

        // Create a basic target topology for the remote container
        final Topology basicTargetTopology = TopologyServiceImpl_AttachContainerToTest.createBasicTargetTopology();
        final Subdomain targetSubdomain = basicTargetTopology.getDomain().getSubDomain().get(0);
        targetSubdomain.setMode(SubdomainMode.DYNAMIC);
        final Properties targetServerProps = this.targetServerPropertiesBuilder
                .createMinimalServerProperties(TARGET_CONTAINER_NAME);
        final ConfigurationService targetCfgSvc = new ConfigurationServiceImpl();
        ConfigurationTestUtils.startConfigurationService(targetCfgSvc);
        try {
            targetCfgSvc.loadConfiguration(targetServerProps, basicTargetTopology);
            final PetalsCompositeController targetPetalsCompositeCtrl = EasyMock
                    .createMock(PetalsCompositeController.class);
            EasyMock.expect(targetPetalsCompositeCtrl.getRegistryConfiguration(TARGET_PASS_PHRASE))
                    .andReturn(new Properties()).anyTimes();
            EasyMock.replay(targetPetalsCompositeCtrl);
            final TopologyService targetTopoSvc = new TopologyServiceImpl();
            ReflectionHelper.setPrivateField(targetTopoSvc, "configurationService", targetCfgSvc);
            ReflectionHelper.setPrivateField(targetTopoSvc, "petalsCompositeController", targetPetalsCompositeCtrl);
            this.startTopologyService(targetTopoSvc);
            try {
                // Create a basic sample topology for the current container
                final Topology basicCurrentTopology = ConfigurationTestUtils.createBasicSampleTopology();
                final Subdomain currentSubdomain = basicCurrentTopology.getDomain().getSubDomain().get(0);
                currentSubdomain.setMode(SubdomainMode.DYNAMIC);

                // Create and initialize the configuration service
                final Properties currentServerProps = this.localServerPropertiesBuilder
                        .createMinimalServerProperties(ConfigurationTestUtils.CONTAINER_NAME_0);
                final ConfigurationService currentCfgSvc = new ConfigurationServiceImpl();
                ConfigurationTestUtils.startConfigurationService(currentCfgSvc);
                try {
                    currentCfgSvc.loadConfiguration(currentServerProps, basicCurrentTopology);

                    final PetalsCompositeController petalsCompositeCtrlMock = EasyMock
                            .createMock(PetalsCompositeController.class);
                    petalsCompositeCtrlMock.stopMessagingSubsystem();
                    petalsCompositeCtrlMock.stopJbiArtefacts();
                    petalsCompositeCtrlMock.startMessagingSubsystem();
                    petalsCompositeCtrlMock.stopRegistryComponent();
                    petalsCompositeCtrlMock.startRegistryComponent();
                    petalsCompositeCtrlMock.recoverSystem();
                    petalsCompositeCtrlMock.freeAllJbiArtifacts();
                    petalsCompositeCtrlMock.onConfigurationUpdated();
                    EasyMock.replay(petalsCompositeCtrlMock);

                    final EndpointRegistry registryMock = EasyMock.createMock(EndpointRegistry.class);
                    registryMock.removeAllLocalEndpoints();
                    EasyMock.expect(registryMock.getDynamicTopology()).andReturn(basicTargetTopology);
                    EasyMock.expect(registryMock.lockDynamicTopology()).andReturn(Boolean.TRUE);
                    registryMock.unlockDynamicTopology();
                    registryMock.setDynamicTopology(basicTargetTopology);
                    EasyMock.replay(registryMock);

                    final TopologyService currentTopoSvc = new TopologyServiceImpl();
                    ReflectionHelper.setPrivateField(currentTopoSvc, "configurationService", currentCfgSvc);
                    ReflectionHelper.setPrivateField(currentTopoSvc, "registry", registryMock);
                    ReflectionHelper.setPrivateField(currentTopoSvc, "petalsCompositeController",
                            petalsCompositeCtrlMock);
                    this.startTopologyService(currentTopoSvc);

                    try {
                        final ContainerConfiguration targetContainerConfiguration = new ContainerConfiguration();
                        targetContainerConfiguration.setName(TARGET_CONTAINER_NAME);
                        targetContainerConfiguration.setSubdomainName(TARGET_SUBDOMAIN_NAME);
                        targetContainerConfiguration.setHost(TARGET_JMX_HOST);
                        targetContainerConfiguration.setJmxRMIConnectorPort(TARGET_JMX_PORT);

                        final TopologyServiceClient jmxTopologyServiceClientMock = EasyMock
                                .createMock(TopologyServiceClient.class);
                        EasyMock.expect(jmxTopologyServiceClientMock.retrieveTopology(TARGET_PASS_PHRASE))
                                .andReturn(targetTopoSvc.retrieveTopology(TARGET_PASS_PHRASE)).anyTimes();
                        EasyMock.expect(jmxTopologyServiceClientMock.retrieveRegistryImplementation())
                                .andReturn(DummyRegistryImpl.class.getName()).anyTimes();
                        jmxTopologyServiceClientMock.addContainerConfiguration(currentCfgSvc
                                .getContainerConfiguration());
                        EasyMock.expectLastCall().anyTimes();
                        EasyMock.replay(jmxTopologyServiceClientMock);
                        this.jmxApiOnTargetContainer.setTopologyServiceClient(jmxTopologyServiceClientMock);

                        currentTopoSvc.attachContainerTo(targetContainerConfiguration, TARGET_PASS_PHRASE);
                    } finally {
                        this.stopTopologyService(currentTopoSvc);
                    }
                } finally {
                    ConfigurationTestUtils.stopConfigurationService(currentCfgSvc);
                }
            } finally {
                this.stopTopologyService(targetTopoSvc);
            }
        } finally {
            ConfigurationTestUtils.stopConfigurationService(targetCfgSvc);
        }
    }

    /**
     * Create a sample topology with only required parameters
     * 
     * @return A sample topology with only required parameters
     */
    private static Topology createBasicTargetTopology() {
        final Topology topology = new Topology();
        final Domain domain = new Domain();
        topology.setDomain(domain);

        domain.setName(TARGET_DOMAIN_NAME);

        final Subdomain subdomain = new Subdomain();
        domain.getSubDomain().add(subdomain);
        subdomain.setName(TARGET_SUBDOMAIN_NAME);
        subdomain.setMode(SubdomainMode.DYNAMIC);

        final Container container0 = new Container();
        subdomain.getContainer().add(container0);
        container0.setName(TARGET_CONTAINER_NAME);
        container0.setHost(TARGET_JMX_HOST);
        container0.setUser(TARGET_JMX_USER);
        container0.setPassword(TARGET_JMX_PWD);

        final JmxService jmxService0 = new JmxService();
        container0.setJmxService(jmxService0);

        final TransportService transportService0 = new TransportService();
        transportService0.setTcpPort(TARGET_CONTAINER_TCP_PORT);
        container0.setTransportService(transportService0);

        return topology;
    }

}
